import 'dart:convert';
import 'package:fis_common/logger/logger.dart';
import 'package:get/get.dart';
import 'package:vitalapp/architecture/services/default_entry_setting.dart';
import 'package:vitalapp/architecture/storage/text_storage.dart';

/// 默认词条服务
class DefaultEntryService extends GetxService {
  String _cache = "";

  /// 获取设置的缓存
  void restoreLocalCache() {
    try {
      var map = json.decode(_cache);
      DefaultEntrySetting.ins.setDataFromJson(map);
    } catch (e) {
      logger.e('DefaultEntryService restoreLocalCache ex:', e);
    }
  }

  static const _SETTING_FILENAME = "local_default_entry_setting.conf";
  TextStorage _entryStorage = TextStorage(fileName: _SETTING_FILENAME);

  ///初始化
  Future<void> init() async {
    // 创建实例
    final list = await _entryStorage.read();
    if (list == null) {
      _cache = "";
    } else {
      _cache = list;
      restoreLocalCache();
    }
  }

  ///保存配置
  Future<bool> save() async {
    var jsonStr = json.encode(DefaultEntrySetting.ins.toJson());
    return _entryStorage.save(jsonStr);
  }
}
