import 'dart:convert';

import 'package:fis_jsonrpc/rpc.dart';
import 'package:vitalapp/managers/interfaces/exam.dart';
import 'package:vitalapp/rpc.dart';
import 'package:vitalapp/store/store.dart';
import 'package:fis_common/logger/logger.dart';

class ExamManager implements IExamManager {
  @override
  Future<bool?> createExam(CreateExamRequest examRequest) async {
    try {
      examRequest.token = Store.user.token;
      final examCode = await rpc.vitalExam.createExamAsync(
        examRequest,
      );
      print(jsonEncode(examRequest.toJson()));
      if (examCode.isNotEmpty) {
        return true;
      }
      return false;
    } catch (e) {
      print(jsonEncode(examRequest.toJson()));

      logger.e("ExamManager sync user info error.", e);
      return null;
    }
  }

  @override
  Future<bool?> editExam(UpdateExamRequest examRequest) async {
    try {
      examRequest.token = Store.user.token;
      final result = await rpc.vitalExam.updateExamAsync(
        examRequest,
      );
      return result;
    } catch (e) {
      logger.e("ExamManager sync user info error.", e);
      return null;
    }
  }

  @override
  Future<bool?> updateExamByBatchNumberAsync(
      UpdateExamByBatchNumberRequest updateExamByBatchNumberRequest) async {
    try {
      updateExamByBatchNumberRequest.token = Store.user.token;
      final result = await rpc.vitalExam.updateExamByBatchNumberAsync(
        updateExamByBatchNumberRequest,
      );
      print(jsonEncode(updateExamByBatchNumberRequest.toJson()));
      return result;
    } catch (e) {
      logger.e("updateExamByBatchNumberAsync sync user info error.", e);
      return null;
    }
  }

  @override
  Future<List<ExamDTO>?> getExamByBatchNumberAsync(String batchNumber) async {
    GetExamByBatchNumberRequest getExamByBatchNumberRequest =
        GetExamByBatchNumberRequest();
    try {
      getExamByBatchNumberRequest.token = Store.user.token;
      getExamByBatchNumberRequest.batchNumber = batchNumber;
      final result = await rpc.vitalExam.getExamByBatchNumberAsync(
        getExamByBatchNumberRequest,
      );
      print(jsonEncode(getExamByBatchNumberRequest.toJson()));
      return result;
    } catch (e) {
      logger.e("updateExamByBatchNumberAsync sync user info error.", e);
      return null;
    }
  }

  @override
  Future<List<ExamConclusionDTO>?> getPatientExamByPageAsync(
      String patientCode) async {
    try {
      GetPatientExamByPageRequest request = GetPatientExamByPageRequest(
        patientCode: patientCode,
        token: Store.user.token,
        examKey: "HEITCMC",
        pageIndex: 1,
        pageSize: 100,
      );
      print(jsonEncode(request.toJson()));
      final result = await rpc.vitalExam.getPatientExamByPageAsync(request);
      return result.pageData;
    } catch (e) {
      logger.e("getPatientExamByPageAsync sync user info error.", e);
      return null;
    }
  }
}
