import 'package:fis_jsonrpc/rpc.dart';
import 'package:vitalapp/managers/interfaces/models/patient_model_dto.dart';
import 'package:vitalapp/pages/patient/info/widgets/ext_model.dart';

import 'base.dart';

abstract class IPatientManager implements IManager {
  /// 创建居民
  ///
  /// 返回 Code
  Future<String?> create(CreatePatientRequest2 request);

  /// 获取居民分页列表
  Future<PageCollection<PatientModelDTO>?> getPagedList(
      PatientPageRequest request);

  /// 获取离线居民列表
  Future<List<PatientModelDTO>?> getPagedOfflineList(
    String? keyword,
    DateTime? startTime,
    DateTime? endTime,
  );

  ///创建病人扩展信息
  Future<String> createPatientExtension(
      String patientCode,
      PatientHealthInfoModel healthInfo,
      PatientDetailInfoModel detailInfo,
      String healthCode);

  /// 获取居民详情
  Future<PatientDTO?> getDetail(String code);

  Future<bool> updatePatientAsync(UpdatePatientRequest2 request);

  ///创建居民健康信息
  Future<String> createPatientExtensionAsync(
      CreatePatientExtensionRequest request);

  ///获取居民健康信息
  Future<PatientExtensionDTO?> getPatientExtensionDetailByCodeAndKeyAsync(
      GetPatientExtensionByCodeAndKeyRequest request);

  ///更新居民健康信息
  Future<bool> updatePatientExtensionAsync(
      UpdatePatientExtensionRequest request);

  ///设置居民的人群分类
  Future<bool> setCrowdLabelsAsync(
      String patientCode, List<String> carowLabels);

  ///获取所有userCode等于空的数据，并赋值
  ///
  ///[userCode] 用户Code
  ///
  Future<void> resettingUsercodeIsEmptyData();

  /// 切换当前选中居民
  Future<void> switchCurrentPatient(PatientDTO dto);

  /// 根据code切换当前选中居民
  Future<void> switchCurrentPatientByCode(String code);

  /// 根据体检号获取体检者信息
  Future<RegisterPersonInfoDTO?>
      getRegisterPersonInfoByPhysicalExamNumberAsync({
    required String physicalExamNumber,
  });

  // /// 更新离线记录为同步成功
  // Future<void> updateOfflineSyncDone();

  /// 删除居民
  Future<bool> removePatient(String code);

  /// 获取统计信息
  Future<PatientStatisticDTO> getStatistic();
}
