import 'dart:convert';

import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:fis_common/event/event_type.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:get/get.dart';
import 'package:vitalapp/components/appbar.dart';
import 'package:vitalapp/components/table/table_column.dart';
import 'package:vitalapp/managers/interfaces/exam.dart';
import 'package:vitalapp/pages/check/examination/controller.dart';
import 'package:vitalapp/pages/check/examination/view.dart';
import 'package:vitalapp/pages/check/health_check_record/controller.dart';
import 'package:vitalapp/pages/medical_checkup_station/registration/state/list.dart';
import 'package:vitalapp/pages/medical_checkup_station/registration/widgets/form/index.dart';
import 'package:vitalapp/pages/medical_checkup_station/usb_print/module/printer_info.dart';
import 'package:vitalapp/pages/medical_checkup_station/usb_print/page/temp/label_temp.dart';

/// 这是体检记录
class ListController {
  late HealthCheckRecordController controller;

  ListController(HealthCheckRecordController controller) {
    this.controller = controller;
  }
  ResidentModel currentResident = ResidentModel(idNumber: "");

  /// 登记列表表头
  List<TableColumn<ResidentModel>> buildTableColumns() {
    var textStyle = TextStyle(
      fontSize: 18,
      overflow: TextOverflow.ellipsis,
    );
    return <TableColumn<ResidentModel>>[
      TableColumn<ResidentModel>(
        headerText: "体检号",
        maxWidth: 150,
        render: (rowData, index) => Container(
          padding: const EdgeInsets.symmetric(vertical: 16),
          child: Text(
            rowData.physicalExamNumber ?? '',
            style: textStyle,
          ),
        ),
      ),
      TableColumn<ResidentModel>(
        headerText: "姓名",
        maxWidth: 100,
        render: (rowData, index) => Text(
          rowData.name ?? '',
          style: textStyle,
        ),
      ),
      TableColumn<ResidentModel>(
        headerText: "年龄",
        maxWidth: 80,
        render: (rowData, index) => Text(
          rowData.age != null ? rowData.age.toString() : "",
          style: textStyle,
        ),
      ),
      TableColumn<ResidentModel>(
        headerText: "身份证号",
        // maxWidth: 200,
        render: (rowData, index) => Center(
          child: Text(
            rowData.idNumber,
            style: textStyle,
          ),
        ),
      ),
      TableColumn<ResidentModel>(
        headerText: "手机号",
        maxWidth: 120,
        render: (rowData, index) => Center(
          child: Text(
            rowData.phone ?? '',
            style: textStyle,
          ),
        ),
      ),
      TableColumn<ResidentModel>(
        headerText: "体检状态",
        maxWidth: 120,
        render: (rowData, index) => Center(
          child: Text(
            rowData.physicalExamStatus ?? "",
            style: textStyle,
          ),
        ),
      ),
      TableColumn<ResidentModel>(
        headerText: "操作",
        maxWidth: 360,
        width: 200,
        render: (rowData, index) => FittedBox(
          child: Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              _buildPrint(rowData),
              _buildHealthCheck(rowData),
            ],
          ),
        ),
      ),
    ];
  }

  /// 体检
  Widget _buildHealthCheck(ResidentModel rowData) {
    return TextButton(
      onPressed: () async {
        final FEventHandler<bool> onSubmitEvent = FEventHandler<bool>();
        Map<String, dynamic> fullMap =
            await handleExamData(rowData.batchNumber);
        Get.lazyPut(
          () => ExaminationController(
            patientCode: rowData.idNumber,
            batchNumber: rowData.batchNumber ?? '',
          ),
        );
        await Get.dialog(
          Scaffold(
            body: ExaminationPage(
              idCard: rowData.idNumber,
              onSubmitEvent: onSubmitEvent,
              examData: jsonEncode(fullMap),
            ),
            appBar: VAppBar(
              titleText: "体检",
            ),
          ),
        );
      },
      child: const Text(
        "体检表",
        style: TextStyle(fontSize: 18),
      ),
    );
  }

  /// 打印按钮
  Widget _buildPrint(ResidentModel rowData) {
    return TextButton(
      onPressed: () {
        if (kIsWeb) {
          controller.getExamLabelsByExamNoAsync(rowData);
        } else {
          Get.dialog(
            Center(
              child: Stack(
                children: [
                  ScreenUtilInit(
                    designSize: const Size(360 * 1.2, 240 * 1.2),
                    scaleByHeight: true,
                    builder: (BuildContext context, Widget? child) {
                      return LabelTempWidget(
                        controller.printInfo ?? PrinterInfo(),
                        labelWidth: 360,
                        labelHeight: 240,
                        residentModel: rowData,
                      );
                    },
                  ),
                  Positioned(
                    right: 66,
                    child: IconButton(
                      onPressed: () {
                        Get.back();
                      },
                      icon: Icon(
                        Icons.close,
                        size: 40,
                      ),
                    ),
                  ),
                ],
              ),
            ),
          );
        }
      },
      child: const Text(
        "打印标签",
        style: TextStyle(fontSize: 18),
      ),
    );
  }

  /// 编辑按钮
  Widget _buildEditRegistration(ResidentModel rowData) {
    return TextButton(
      onPressed: () async {
        final PatientDTO? patient = PatientDTO(
          patientName: rowData.name,
          phone: rowData.phone,
          patientAddress: rowData.homeAddress,
          cardNo: rowData.idNumber,
          code: rowData.code,
          birthday: rowData.birthDay,
          patientGender: rowData.sex == "男"
              ? GenderEnum.Male
              : rowData.sex == "女"
                  ? GenderEnum.Female
                  : GenderEnum.Unknown,
        );
        final PatientDTO? result = await Get.dialog<PatientDTO>(
          RegistrationFormDialog(
            patient: patient,
            isEdit: true,
          ),
          barrierDismissible: false,
        );
        //controller.editResident(result);
        print(result);
      },
      child: const Text(
        "编辑",
        style: TextStyle(fontSize: 18),
      ),
    );
  }

  /// 处理体检数据
  Future<Map<String, dynamic>> handleExamData(String? batchNumber) async {
    final _examManager = Get.find<IExamManager>();
    List<String> stringList = [];
    List<ExamDTO>? examList =
        await _examManager.getExamByBatchNumberAsync(batchNumber ?? '');
    examList?.forEach((element) {
      stringList.add(element.examData ?? "{}");
    });
    Map<String, dynamic> fullMap = {};

    stringList.forEach((str) {
      Map<String, dynamic> map = jsonDecode(str);
      fullMap.addAll(map);
    });
    return fullMap;
  }
}
