import 'dart:convert';

import 'package:fis_jsonrpc/rpc.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:vitalapp/architecture/defines.dart';
import 'package:vitalapp/architecture/utils/prompt_box.dart';
import 'package:vitalapp/global.dart';
import 'package:vitalapp/managers/interfaces/exam.dart';
import 'package:vitalapp/managers/interfaces/follow_up.dart';
import 'package:vitalapp/pages/check/follow_up/controller.dart';
import 'package:vitalapp/pages/check/traditional_chinese_medicine_consitution_record/state.dart';
import 'package:vitalapp/pages/traditional_chinese_medicine_constitution/widget/ConfigurableCard.dart';
import 'package:vitalapp/store/store.dart';

class TraditionalChineseMedicineConsitutionRecordController
    extends FControllerBase {
  TraditionalChineseMedicineConsitutionRecordController();
  final state = TraditionalChineseMedicineConsitutionRecordState();
  final _followUpManager = Get.find<IFollowUpManager>();
  final _examManager = Get.find<IExamManager>();
  late String patientCode;
  late String patientName;
  // late FollowUpController _followUpController;
  late String key;
  @override
  void onReady() {
    super.onReady();
    _initData();
  }

  _initData() {
    update(["contract_records"]);
    patientCode = Store.user.currentSelectPatientInfo?.code ?? '';
    patientName = Store.user.currentSelectPatientInfo?.patientName ?? '';
    final p = Get.parameters;
    key = p["key"] ?? "";
    getPatientExamList();
    // _followUpController = Get.put(FollowUpController());
  }

  Future<void> getPatientExamList() async {
    try {
      var result = await _examManager.getPatientExamByPageAsync(patientCode);
      if (result != null) {
        state.examConclusionDTO = result;
      }
    } catch (e) {
      return;
    }
  }

  Future<void> updateExamAsync(key, code, data) async {
    if (!kIsOnline) {
      // TODO: 暂不支持离线体检
      PromptBox.toast("请检查网络连接");
      return;
    }
    var request = UpdateExamRequest(
      key: "HEITCMC",
      examData: data,
      code: code,
    );
    print(jsonEncode(request.toJson()));
    final result = await _examManager.editExam(request);
    if (result ?? false) {
      try {
        PromptBox.toast('保存成功');
      } catch (err) {
        PromptBox.toast('保存失败');
      }
    }
  }

  Map<String, dynamic> storeTypeMap = {
    "Qi_Score": '气虚质',
    "Yang_Score": '阳虚质',
    "Yin_Score": '阴虚质',
    "Tan_Score": '痰湿质',
    "Shi_Score": '湿热质',
    "Xue_Score": '血瘀质',
    "Qiyu_Score": '气郁质',
    "Te_Score": '特禀质',
    "Ping_Score": '平和质',
  };

  String getConstitutionalType(String constitutionalType) {
    return storeTypeMap[constitutionalType];
  }

  Future<List> getFollowUpTableData(FollowUpRecordDTO dto, int index) async {
    var currentFollowUp = dto.followUpRecordDatas?[index];
    var followUpTableData = await _followUpManager
        .getFollowUpRecordListByYearAsync(GetFollowUpRecordListByYearRequest(
      year: (currentFollowUp?.followUpTime?.year ?? DateTime.now().year)
          .toString(),
      keys: [currentFollowUp?.key ?? ''],
      patientCode: patientCode,
    ));
    List tableData = [];

    followUpTableData?.first.followUpRecordDatas?.forEach((element) {
      Map followUp = {
        "Follow_Time": element.followUpTime != null
            ? DateFormat("yyyy-MM-dd").format(element.followUpTime!.toLocal())
            : "",
        "Follow_Type": '${(element.followUpMode?.index ?? 0) + 1} ',
        "Next_Follow_Up_Time": element.nextFollowUpTime != null
            ? DateFormat("yyyy-MM-dd")
                .format(element.nextFollowUpTime!.toLocal())
            : "",
      };
      followUp.addAll(jsonDecode(element.followUpData ?? ''));
      tableData.add(followUp);
    });

    return tableData;
  }

  toCheckPage(ExamConclusionDTO dataDTO) async {
    Get.to(
      ConfigurableCard(
        cardKey: "LNRZYYJKGLFWJL",
        examData: dataDTO.examData,
        patientCode: patientCode,
        isEdit: true,
        callBack: (key, code, data) async {
          await updateExamAsync(key, dataDTO.code, data);
          await getPatientExamList();
          return true;
        },
      ),
      transition: Transition.rightToLeft,
    );
  }
}
