import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:vitalapp/components/appbar.dart';
import 'package:vitalapp/consts/styles.dart';
import 'package:vitalapp/pages/check/traditional_chinese_medicine_consitution_record/controller.dart';
import 'package:vitalapp/pages/traditional_chinese_medicine_constitution/view.dart';
import 'package:vitalapp/pages/widgets/record_common_item.dart';

class TraditionalChineseMedicineConsitutionRecordPage
    extends GetView<TraditionalChineseMedicineConsitutionRecordController> {
  const TraditionalChineseMedicineConsitutionRecordPage({Key? key})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return GetBuilder(
        init: TraditionalChineseMedicineConsitutionRecordController(),
        id: "TraditionalChineseMedicineConsitutionRecordPageRecord",
        builder: (_) {
          return Scaffold(
            backgroundColor: const Color.fromRGBO(238, 238, 238, 1),
            appBar: VAppBar(
              titleWidget: const Text('中医体质记录'),
              actions: [
                IconButton(
                  onPressed: () async {
                    await Get.dialog(
                        TraditionalChineseMedicineConstitutionView());
                    controller.getPatientExamList();
                  },
                  icon: Row(
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: [
                      Text(
                        "检测",
                        style: TextStyle(fontSize: 28, color: Colors.white),
                      ),
                      Icon(Icons.add, size: 40),
                    ],
                  ),
                ),
              ],
            ),
            body: Stack(
              children: [
                Row(
                  mainAxisAlignment: MainAxisAlignment.start,
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    _buildDiagram(),
                    _buildListView(),
                  ],
                )
              ],
            ),
          );
        });
  }

  Widget _buildDiagram() {
    return Expanded(
      flex: 1,
      child: Padding(
        padding: const EdgeInsets.all(16.0).copyWith(right: 0),
        child: Container(
          // color: Colors.white,
          padding: const EdgeInsets.all(16),
          decoration: BoxDecoration(
            color: Colors.white,
            border: Border.all(
              color: Colors.white,
            ),
            borderRadius: GlobalStyles.borderRadius,
          ),
          child: Image.asset(
            'assets/images/exam/normalMeasurementChart.png',
            height: double.infinity,
            fit: BoxFit.fitWidth,
          ),
        ),
      ),
    );
  }

  Widget _buildListView() {
    return Expanded(
        flex: 2,
        child: Padding(
          padding: const EdgeInsets.all(16),
          child: RefreshIndicator(
              child: Obx(
                () {
                  final list = controller.state.examConclusionDTO;
                  final children = <Widget>[];
                  for (int index = 0; index < list.length; index++) {
                    children.add(_followUpRecordCard(
                      index: index,
                      dto: list[index],
                    ));
                  }
                  return list.isEmpty
                      ? Container(
                          margin: const EdgeInsets.only(top: 80),
                          child: Column(
                            children: [
                              Center(
                                child: Image.asset(
                                  "assets/images/no_data.png",
                                  width: 300,
                                  height: 300,
                                  fit: BoxFit.cover,
                                ),
                              ),
                              const Text(
                                "暂无数据，先看看别的吧",
                                style: TextStyle(fontSize: 18),
                              ),
                            ],
                          ),
                        )
                      : GridView(
                          gridDelegate:
                              const SliverGridDelegateWithFixedCrossAxisCount(
                            crossAxisCount: 1,
                            mainAxisSpacing: 16,
                            crossAxisSpacing: 20,
                            childAspectRatio: 900 / 150,
                          ),
                          children: children,
                        );
                },
              ),
              onRefresh: () async {}),
        ));
  }
}

// ignore: camel_case_types
class _followUpRecordCard extends StatelessWidget {
  final ExamConclusionDTO dto;
  final int index;
  _followUpRecordCard({
    required this.dto,
    required this.index,
  });
  final controller =
      Get.find<TraditionalChineseMedicineConsitutionRecordController>();
  @override
  Widget build(BuildContext context) {
    final body = Stack(
      children: [
        Row(
          children: [
            Expanded(
              flex: 10,
              child: Container(
                padding: const EdgeInsets.symmetric(
                  horizontal: 30,
                  vertical: 12,
                ),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    const SizedBox(
                      height: 8,
                    ),
                    const SizedBox(
                      height: 20,
                    ),
                    Wrap(
                      alignment: WrapAlignment.start,
                      spacing: 20,
                      runSpacing: 8,
                      children: [
                        SizedBox(
                          width: 300,
                          child: RecordCommonItem(
                            itemName: '姓名',
                            itemValue: dto.patientName ?? "",
                            fontSize: 18,
                          ),
                        ),
                        RecordCommonItem(
                          itemName: '体质结果',
                          itemValue: controller
                              .getConstitutionalType(dto.tcmConclusion ?? ''),
                          fontSize: 18,
                        ),
                      ],
                    ),
                    const SizedBox(
                      height: 20,
                    ),
                    Wrap(
                      alignment: WrapAlignment.start,
                      spacing: 20,
                      runSpacing: 8,
                      children: [
                        SizedBox(
                          width: 300,
                          child: RecordCommonItem(
                            itemName: '医生',
                            itemValue: dto.contractedDoctor ?? "",
                            fontSize: 18,
                          ),
                        ),
                        RecordCommonItem(
                          itemName: '时间',
                          itemValue: dto.updateTime != null
                              ? DateFormat("yyyy-MM-dd HH:mm:ss")
                                  .format(dto.updateTime!.toLocal())
                              : "",
                          fontSize: 18,
                        ),
                      ],
                    )
                  ],
                ),
              ),
            ),
            Expanded(
              child: Icon(
                Icons.keyboard_arrow_right,
                size: 64,
                color: Colors.grey.shade400,
              ),
            )
          ],
        ),
      ],
    );
    return Material(
      borderRadius: GlobalStyles.borderRadius,
      child: Ink(
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: GlobalStyles.borderRadius,
        ),
        child: InkWell(
          borderRadius: GlobalStyles.borderRadius,
          onTap: () {
            controller.toCheckPage(dto);
          },
          child: body,
        ),
      ),
    );
  }
}
