import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_ui/base_define/page.dart';
import 'package:get/get.dart';
import 'package:vitalapp/pages/consultation_record_view/index.dart';
import 'package:vitalapp/pages/consultation_record_view/widgets/search_input.dart';

class ConsultationRecordFilter extends GetView<ConsultationRecordViewController>
    implements FPage {
  ConsultationRecordFilter({Key? key}) : super(key: key);

  @override
  String get pageName => 'ConsultationRecordFilter';

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: EdgeInsets.symmetric(vertical: 10, horizontal: 15),
      child: Column(
        children: [
          Row(
            children: [
              Container(
                width: 280,
                child: SearchInput(
                  businessParent: this,
                  margin: EdgeInsets.all(0),
                  onChanged: (value) {
                    controller.keyWord = value;
                  },
                  hintText: i18nBook.remedical.searchKeyword.t,
                  focusNode: controller.searchInputFocusNode,
                  serchOnPressed: () {
                    controller.searchFindRecordPages();
                  },
                ),
              ),
              Expanded(child: FSizedBox()),
              GetBuilder<ConsultationRecordViewController>(
                id: "record_filter_tab",
                builder: (_) {
                  return FContainer(
                    margin: EdgeInsets.only(left: 15),
                    child: FRow(
                      children: [
                        _tabRadio(
                          title: i18nBook.common.all.t,
                          value: RecordProcessStateEnum.All,
                        ),
                        _tabRadio(
                          title: i18nBook.remedical.pending.t,
                          value: RecordProcessStateEnum.Wait,
                        ),
                        _tabRadio(
                          title: i18nBook.remedical.completed.t,
                          value: RecordProcessStateEnum.Done,
                        ),
                      ],
                    ),
                  );
                },
              ),
              Expanded(child: FSizedBox()),
              InkWell(
                onTap: () {},
                child: FIcon(
                  Icons.file_upload,
                  color: const Color(0xff2c77e5),
                ),
              ),
              const SizedBox(width: 10),
              InkWell(
                onTap: () {
                  controller.refreshData();
                },
                child: FIcon(
                  Icons.refresh,
                  color: const Color(0xff2c77e5),
                ),
              ),
              const SizedBox(width: 10),
              GetBuilder<ConsultationRecordViewController>(
                id: "record_more_filter_icon",
                builder: (_) {
                  return FIconButton(
                    businessParent: this,
                    name: i18nBook.common.filter.t,
                    onPressed: () {
                      controller.openOrCloseMoreFilter();
                      controller.scaffoldKey.currentState?.openEndDrawer();
                    },
                    icon: FIcon(
                      Icons.filter_alt_rounded,
                      color: const Color(0xff2c77e5),
                    ),
                  );
                },
              ),
            ],
          ),
        ],
      ),
    );
  }

  FWidget _tabRadio({
    required String title,
    required RecordProcessStateEnum value,
  }) {
    return FInkWell(
      onTap: () {
        controller.changeTypeFilter(value);
      },
      child: FContainer(
        margin: EdgeInsets.only(right: 15),
        child: FRow(
          children: [
            FRadio(
              value: value,
              groupValue: controller.typeFilter,
              onChanged: (v) {
                controller.changeTypeFilter(value);
              },
            ),
            FText(
              title,
              style: TextStyle(
                color: controller.typeFilter == value
                    ? const Color(0xff2c77e5)
                    : const Color(0xff4c4948),
              ),
            ),
          ],
        ),
      ),
    );
  }
}
