import 'package:fis_i18n/i18n.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:fis_ui/base_define/page.dart';
import 'package:fis_ui/index.dart';
import 'package:fis_jsonrpc/rpc.dart';

import 'package:vitalapp/managers/interfaces/models/selected_model.dart';
import 'package:vitalapp/pages/consultation_record_view/controller.dart';
import 'package:vitalapp/pages/consultation_record_view/widgets/time_filter.dart';

class RecordFilterDrawer extends StatefulWidget implements FPage {
  @override
  State<StatefulWidget> createState() {
    return _RecordFilterDrawerState();
  }

  @override
  String get pageName => "RecordFilterDrawer";
}

class _RecordFilterDrawerState extends State<RecordFilterDrawer> {
  final controller = Get.find<ConsultationRecordViewController>();
  final GlobalKey _startKey = GlobalKey();
  final GlobalKey _endKey = GlobalKey();
  static MaterialStateProperty<EdgeInsetsGeometry> paddingEdgeInsets =
      MaterialStateProperty.all(
    EdgeInsets.symmetric(
      vertical: 15,
      horizontal: 50,
    ),
  );
  @override
  Widget build(BuildContext context) {
    return Column(
      children: [
        _buildFilterClose(),
        _buildFilterTime(context),
        _buildFilterStatus(context),
        FExpanded(child: FSizedBox()),
        _buildFilterButtonList(),
        FSizedBox(
          height: 15,
        ),
      ],
    );
  }

  FWidget _buildFMutiSelectFilter({
    required String title,
    required List<FMutiSelectModel> selectSource,
    required TextEditingController textEditingController,
    required void Function() onTap,
  }) {
    return _buildRowTitle(
      title: title,
      rowWidget: FBorderInput(
        hintSize: 16,
        contentSize: 16,
        maxLength: 20,
        width: 340,
        borderColor: Color.fromRGBO(234, 234, 234, 1),
        height: 38,
        controller: textEditingController,
        mouseCursor: SystemMouseCursors.click,
        readOnly: true,
        onTap: onTap,
      ),
      isExpand: false,
    );
  }

  FWidget _buildRowTitle(
      {required String title,
      required FWidget rowWidget,
      bool isExpand = true}) {
    return FContainer(
      height: 40,
      margin: EdgeInsets.only(bottom: 5),
      child: FRow(
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          FContainer(
            width: 150,
            padding: EdgeInsets.only(left: 30),
            child: FText(
              title,
              overflow: TextOverflow.ellipsis,
              maxLines: 2,
            ),
          ),
          if (isExpand) ...[
            FExpanded(
              child: rowWidget,
            )
          ] else ...[
            rowWidget,
          ]
        ],
      ),
    );
  }

  List<double> _findRenderObject(GlobalKey key) {
    RenderBox? renderBox = key.currentContext!.findRenderObject() as RenderBox;
    // offset.dx , offset.dy 就是控件的左上角坐标
    renderBox.localToGlobal(Offset.zero);
    var offset = renderBox.localToGlobal(Offset.zero);
    return [offset.dx, offset.dy + 38];
  }

  FWidget _buildTimeInputSelect({
    required DateTime dateTime,
    required String keyString,
    required ValueCallback valueCallback,
    required BuildContext context,
    required GlobalKey globalKey,
  }) {
    void selectInput() {
      var timePosition = _findRenderObject(globalKey);
      showDialog(
        context: context,
        barrierDismissible: false,
        builder: (BuildContext context) {
          return TimeSelectDialog(
            globalKey: keyString,
            position: timePosition,
            selectedfn: (datetime) {
              valueCallback.call(datetime);
            },
            startDateTime: controller.startDateTime,
            endDateTime: controller.endDateTime,
          );
        },
      );
    }

    return FBorderInput(
      key: globalKey,
      hintSize: 16,
      contentSize: 16,
      maxLength: 20,
      width: 340,
      borderColor: Color.fromRGBO(234, 234, 234, 1),
      controller: TextEditingController(
        text: DateFormat('yyyy-MM-dd')
            .format(
              dateTime,
            )
            .toString(),
      ),
      onTap: () => selectInput(),
      readOnly: true,
      suffixIcon: FMaterial(
        color: Colors.transparent,
        child: FIconButton(
          businessParent: widget,
          name: "endTimePosition",
          onPressed: () => selectInput(),
          icon: FIcon(
            Icons.date_range_rounded,
          ),
        ),
      ),
      height: 38,
      hintText: i18nBook.remedical.selectEndTime.t,
      onChanged: (value) {},
    );
  }

  FWidget _buildFilterClose() {
    return FSizedBox(
      height: Get.height * 0.08,
      child: FRow(
        mainAxisAlignment: MainAxisAlignment.end,
        children: [
          FIconButton(
            icon: FIcon(Icons.close),
            onPressed: () {
              controller.openOrCloseMoreFilter();
              controller.scaffoldKey.currentState?.closeEndDrawer();
            },
            businessParent: widget,
            name: 'close filter',
          )
        ],
      ),
    );
  }

  FWidget _buildFilterStatus(BuildContext context) {
    return FColumn(
      children: [
        const FSizedBox(height: 15),
        _buildRowTitle(
          title: i18nBook.common.status.t,
          rowWidget: FSelect<FSelectOptionModel, RecordQueryStateEnum>(
            source: controller.recordStatusList,
            hintText: i18nBook.common.status.t,
            optionLabelExtractor: (data) => data.title,
            optionValueExtractor: (data) => data.value,
            value: controller.selectRecordStatus,
            textColor: Colors.black,
            onSelectChanged: (value, index) {
              controller.selectRecordStatus = value!;
            },
          ),
        ),
      ],
    );
  }

  Widget _buildFilterTime(BuildContext context) {
    return GetBuilder<ConsultationRecordViewController>(
      id: "record_time_filter",
      builder: (_) {
        return FColumn(
          children: [
            _buildRowTitle(
              title: i18nBook.realTimeConsultation.startTime.t,
              rowWidget: _buildTimeInputSelect(
                keyString: '_startKey',
                globalKey: _startKey,
                dateTime: controller.startDateTime,
                context: context,
                valueCallback: (value) {
                  controller.startDateTime = value;
                  controller.update(['record_time_filter']);
                  // setState(() {});
                },
              ),
              isExpand: false,
            ),
            const FSizedBox(height: 15),
            _buildRowTitle(
              title: i18nBook.realTimeConsultation.endTime.t,
              rowWidget: _buildTimeInputSelect(
                keyString: '_endKey',
                globalKey: _endKey,
                dateTime: controller.endDateTime,
                context: context,
                valueCallback: (value) {
                  controller.endDateTime = value;
                  controller.update(['record_time_filter']);
                  // setState(() {});
                },
              ),
              isExpand: false,
            ),
          ],
        );
      },
    );
  }

  Widget _buildFilterButtonList() {
    return Container(
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          _buildTextButton(
            i18nBook.common.search.t,
            () async {
              await controller.searchFindRecordPages();
            },
          ),
          FSizedBox(
            width: 15,
          ),
          _buildTextButton(
            i18nBook.measure.resetTone4Btn.t,
            () async {
              await controller.resetFilter();
            },
          ),
        ],
      ),
    );
  }

  /// 普通按钮
  Widget _buildTextButton(
    String textString,
    Function onPressed, {
    MaterialStateProperty<Size?>? fixedSize,
  }) {
    return ElevatedButton(
      onPressed: () => onPressed.call(),
      child: FText(
        textString,
        style: TextStyle(color: Colors.white),
      ),
      style: ButtonStyle(
        padding: MaterialStateProperty.all(
          EdgeInsets.symmetric(
            horizontal: 15,
          ),
        ),
        fixedSize: fixedSize ?? MaterialStateProperty.all(Size(210, 40)),
      ),
    );
  }
}
