import 'dart:async';

import 'package:fis_common/js_plateform/js_platform.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

import 'package:vitalapp/architecture/defines.dart';
import 'package:vitalapp/architecture/network_connectivity.dart';
import 'package:vitalapp/architecture/utils/prompt_box.dart';
import 'package:vitalapp/components/appbar.dart';
import 'package:vitalapp/global.dart';
import 'package:vitalapp/helper/goto_helper.dart';
import 'package:vitalapp/managers/interfaces/account.dart';
import 'package:vitalapp/managers/interfaces/device.dart';
import 'package:vitalapp/managers/interfaces/patient.dart';
import 'package:vitalapp/pages/controllers/home_nav_mixin.dart';
import 'package:vitalapp/pages/home/models/menu.dart';
import 'package:vitalapp/pages/medical/widgets/twelve_ecg.dart';
import 'package:vitalapp/store/store.dart';
import 'package:vnote_device_plugin/consts/types.dart';

import 'state.dart';

class HomeController extends FControllerBase with HomeNavMixin {
  static const String _windowName = "Main";

  /// 仅支持在线模式的菜单路由
  static final _onlineOnlyMenuRouteNames = <String>[
    "/contract/package_list",
    "/check/form",
  ];
  static Widget _buildImgIcon(String assetName) {
    return ClipRect(
      child: SizedBox(
        height: 34,
        child: Image.asset(
          "assets/images/home/$assetName",
          width: 46,
          height: 46,
          fit: BoxFit.fitWidth,
        ),
      ),
    );
  }

  final _patientManager = Get.find<IPatientManager>();

  final state = HomeState();

  List<HomeMenuItem> homeMenuItems = [
    HomeMenuItem(
      key: 'ZY',
      title: "主页",
      routeName: "/dashboard",
      iconData: Icons.home_outlined,
    ),
    HomeMenuItem(
      key: 'XJDA',
      title: "新建档案",
      routeName: "/patient/create",
      // TODO: 图addHealthRecord.png 不太合适，先层叠这样显示
      iconWidget: _PatientAddIconProxyWidget(
        baseIcon: _buildImgIcon('healthRecord.png'),
      ),
    ),
    HomeMenuItem(
      key: 'JKDA',
      title: "健康档案",
      routeName: "/patient/detail",
      iconWidget: _buildImgIcon('healthRecord.png'),
    ),
    HomeMenuItem(
      key: 'JMLB',
      title: "居民列表",
      routeName: "/patient/list",
      iconWidget: _buildImgIcon('patientList.png'),
    ),
    HomeMenuItem(
      key: 'YSQY',
      title: "医生签约",
      routeName: "/contract/package_list",
      iconWidget: _buildImgIcon('doctorSigning.png'),
    ),
    HomeMenuItem(
      key: 'JKTJ',
      title: "健康体检",
      routeName: "/check/form",
      iconWidget: _buildImgIcon('healthCheckup.png'),
    ),
    HomeMenuItem(
      key: 'RQSF',
      title: "人群随访",
      routeName: "/check/follow_up",
      iconWidget: _buildImgIcon('populationFollowUp.png'),
    ),
    HomeMenuItem(
      key: 'JKJC',
      title: "健康检测",
      routeName: "/medical",
      iconWidget: _buildImgIcon('diagnosisDisplay.png'),
      isSelected: true,
    ),
    HomeMenuItem(
      title: "中医体质",
      routeName: "/TraditionalChineseMedicineConstitution",
      key: "ZYTZ",
      iconWidget: _buildImgIcon("medicineConstitution.png"),
      isSelected: true,
    ),
    HomeMenuItem(
      key: 'TJDJ',
      title: "体检列表",
      routeName: "/registrationList",
      iconWidget: _PatientAddIconProxyWidget(
        baseIcon: _buildImgIcon('healthRecord.png'),
      ),
    ),
    HomeMenuItem(
      key: 'TJYY',
      title: "预约管理",
      routeName: "/appointment",
      iconWidget: _PatientAddIconProxyWidget(
        baseIcon: _buildImgIcon('diagnosisDisplay.png'),
      ),
    ),
    HomeMenuItem(
      key: 'TJJCJC',
      title: "基础检查",
      routeName: "/basicCheck",
      iconWidget: _PatientAddIconProxyWidget(
        baseIcon: _buildImgIcon('patientList.png'),
      ),
    ),
    HomeMenuItem(
      key: 'TJXD',
      title: "心电",
      routeName: "/electrocardiogram",
      iconWidget: _PatientAddIconProxyWidget(
        baseIcon: _buildImgIcon('populationFollowUp.png'),
      ),
    ),
    HomeMenuItem(
      key: 'TJNY',
      title: "尿常规",
      routeName: "/ncg",
      iconWidget: _PatientAddIconProxyWidget(
        baseIcon: _buildImgIcon('populationFollowUp.png'),
      ),
    ),
    HomeMenuItem(
      key: 'TJXCG',
      title: "血常规",
      routeName: "/bloodTest",
      iconWidget: _PatientAddIconProxyWidget(
        baseIcon: _buildImgIcon('populationFollowUp.png'),
      ),
    ),
    HomeMenuItem(
      key: 'TJSH',
      title: "生化",
      routeName: "/biochemistryTest",
      iconWidget: _PatientAddIconProxyWidget(
        baseIcon: _buildImgIcon('populationFollowUp.png'),
      ),
    ),
    // HomeMenuItem(
    //   key: 'SZZX',
    //   title: "设置中心",
    //   routeName: "/settings",
    //   iconData: Icons.settings,
    // ),
    HomeMenuItem(
      key: 'TJCS',
      routeName: '/remedicalRecordView',
      title: '超声',
      iconWidget: _PatientAddIconProxyWidget(
        baseIcon: _buildImgIcon('populationFollowUp.png'),
      ),
    ),
    // HomeMenuItem(
    //   key: 'YCBG',
    //   routeName: '/remedicalRecordView',
    //   title: '异常报告',
    //   iconWidget: _PatientAddIconProxyWidget(
    //     baseIcon: _buildImgIcon('populationFollowUp.png'),
    //   ),
    // ),
  ];

  /// 登出
  Future<void> logOut() async {
    final result = await Get.find<IAccountManager>().logout();
    if (result) {
      Get.offAllNamed("/login");
    }
  }

  // Future<void> changeMedical() async {
  //   try {
  //     Get.offAllNamed("/medical", id: 1001);
  //   } catch (e) {}
  // }

  void onlineChanged(_, bool isOnline) {
    state.isOnline = isOnline;
  }

  @override
  void onInit() {
    initMenus();
    super.onInit();
  }

  initAddListener() {
    netChecker.onlineChangedEvent.addListener(onlineChanged);
  }

  netCheckerRemoveListener() {
    netChecker.onlineChangedEvent.removeListener(onlineChanged);
  }

  /// 切换活动菜单
  void switchActiveMenu(HomeMenuItem data) async {
    if (!kIsOnline) {
      // 无网络时，根据仅在线支持的菜单名单判断，进行阻断和提示
      if (_onlineOnlyMenuRouteNames.contains(data.routeName)) {
        PromptBox.toast("请检查网络连接");
        return;
      }
    }
    if (state.currentSelectMenu != data.routeName) {
      switchNavByName(data.routeName);
    }
  }

  void switchNavByName(String name, [Map? patientInfo]) {
    if (Store.user.currentSelectPatientInfo == null) {
      /// 有些路由跳转之前需要判断有没有选择居民
      if (["/patient/detail"].contains(name)) {
        PromptBox.toast("当前未选择居民，请先选择居民后再操作。");
        return;
      }
    }

    state.currentSelectMenu = name;
    NavGotoHelper.goto(name, patientInfo);
  }

  void initMenus() {
    if (Store.user.menuPermissionList?.isNotEmpty ?? false) {
      List<HomeMenuItem> menuItems = [];
      Store.user.menuPermissionList?.forEach((element) {
        for (var item in homeMenuItems) {
          if (item.key == element.code) {
            menuItems.add(item);
          }
        }
      });
      if (!menuItems.any((element) => element.key == "JKJC")) {
        state.currentSelectMenu = menuItems.first.routeName;
      }
      state.menuItems = menuItems;
    } else {
      // state.menuItems = homeMenuItems;
      state.menuItems = [
        HomeMenuItem(
          key: 'XJDA',
          title: "新建档案",
          routeName: "/patient/create",
          // TODO: 图addHealthRecord.png 不太合适，先层叠这样显示
          iconWidget: _PatientAddIconProxyWidget(
            baseIcon: _buildImgIcon('healthRecord.png'),
          ),
        ),
        HomeMenuItem(
          key: 'JMLB',
          title: "居民列表",
          routeName: "/patient/list",
          iconWidget: _buildImgIcon('patientList.png'),
        ),
        HomeMenuItem(
          key: 'JKDA',
          title: "健康档案",
          routeName: "/patient/detail",
          iconWidget: _buildImgIcon('healthRecord.png'),
        ),
        HomeMenuItem(
          key: 'JKJC',
          title: "健康检测",
          routeName: "/medical",
          iconWidget: _buildImgIcon('diagnosisDisplay.png'),
        ),
      ];
    }
  }

  void updateMenus() {
    if (Store.user.menuPermissionList?.isNotEmpty ?? false) {
      List<HomeMenuItem> menuItems = [];
      Store.user.menuPermissionList?.forEach((element) {
        for (var item in homeMenuItems) {
          if (item.key == element.code) {
            menuItems.add(item);
          }
        }
      });

      state.menuItems = menuItems;
    } else {
      state.menuItems = [
        HomeMenuItem(
          key: 'XJDA',
          title: "新建档案",
          routeName: "/patient/create",
          // TODO: 图addHealthRecord.png 不太合适，先层叠这样显示
          iconWidget: _PatientAddIconProxyWidget(
            baseIcon: _buildImgIcon('healthRecord.png'),
          ),
        ),
        HomeMenuItem(
          key: 'JMLB',
          title: "居民列表",
          routeName: "/patient/list",
          iconWidget: _buildImgIcon('patientList.png'),
        ),
        HomeMenuItem(
          key: 'JKDA',
          title: "健康档案",
          routeName: "/patient/detail",
          iconWidget: _buildImgIcon('healthRecord.png'),
        ),
        HomeMenuItem(
          key: 'JKJC',
          title: "健康检测",
          routeName: "/medical",
          iconWidget: _buildImgIcon('diagnosisDisplay.png'),
        ),
        // HomeMenuItem(
        //   key: 'SZZX',
        //   title: "设置中心",
        //   routeName: "/settings",
        //   iconData: Icons.settings,
        // ),
      ];
    }
  }

  void onScanData(String code) async {
    try {
      print('$code');
      final deviceManager = Get.find<IDeviceManager>();
      deviceManager.onScanCode.emit(this, code);
      Get.back();
    } catch (e) {
      print(e);
    }

    // RegisterPersonInfoDTO? registerPersonInfo =
    //     await _patientManager.getRegisterPersonInfoByPhysicalExamNumberAsync(
    //   physicalExamNumber: code,
    // );
    // PatientDTO? patientInfo = PatientDTO();
    // if (registerPersonInfo != null &&
    //     registerPersonInfo.physicalExamNumber != null) {
    //   patientInfo.code = registerPersonInfo.code;
    //   patientInfo.patientName = registerPersonInfo.name;
    //   Store.user.currentSelectRegisterPersonInfo = registerPersonInfo;
    //   Store.user.currentSelectPatientInfo = patientInfo;
    // }
    // Get.back();
    // onScanSwitchPage(state.currentSelectMenu);
    // print(state.currentSelectMenu);
  }

  void onScanSwitchPage(String routeName) {
    switch (routeName) {
      case "/electrocardiogram":
        Get.dialog(_buildMedical());
        return;
    }
  }

  Widget _buildMedical() {
    return Scaffold(
      appBar: VAppBar(
        titleText: "体检心电",
      ),
      body: Container(
        color: Colors.white,
        child: Stack(
          children: [
            Row(
              children: [
                _buildDeviceImage(DeviceTypes.TWELVEHEART),
                _buildMedicalInput(DeviceTypes.TWELVEHEART),
              ],
            ),
            Positioned(
              right: 16,
              bottom: 16,
              child: _buildSaveButton(),
            ),
          ],
        ),
      ),
    );
  }

  Widget _buildMedicalInput(String? currentTab) {
    return Expanded(
      flex: currentTab == DeviceTypes.TWELVEHEART ? 18 : 11,
      child: Stack(
        children: [
          Container(
            padding: const EdgeInsets.all(16),
            child: Column(
              children: [
                _buildContent(),
              ],
            ),
          ),
        ],
      ),
    );
  }

  String _deviceImageUrl(String? currentTab) {
    switch (currentTab) {
      case DeviceTypes.TEMP:
        return 'assets/images/healthCheck/temp.png';
      case DeviceTypes.SUGAR:
        return 'assets/images/healthCheck/sugar.png';
      case DeviceTypes.NIBP:
        return 'assets/images/healthCheck/nibp.png';
      case DeviceTypes.SPO2:
        return 'assets/images/healthCheck/spo2.png';
      case DeviceTypes.WEIGHT:
        return 'assets/images/healthCheck/bmi.png';
      case DeviceTypes.URINE:
        return 'assets/images/healthCheck/urine.png';
      case DeviceTypes.WAIST:
        return 'assets/images/healthCheck/whb.png';
      default:
        return 'assets/images/exam/normalMeasurementChart.png';
    }
  }

  Widget _buildDeviceImage(String? currentTab) {
    if (currentTab == DeviceTypes.TWELVEHEART) {
      return const SizedBox();
    }
    return Expanded(
      flex: 7,
      child: Container(
        alignment: Alignment.topCenter,
        margin: const EdgeInsets.all(16).copyWith(top: 10),
        child: SizedBox(),
      ),
    );
  }

  Widget _buildSaveButton() {
    return Obx(() {
      if (Store.user.currentSelectRegisterPersonInfo == null) {
        return const SizedBox();
      }
      return FloatingActionButton.extended(
        // backgroundColor: Theme.of(context).primaryColor,
        onPressed: () {
          // advanceDebounce(
          //   () => controller.createHeart(
          //     Store.user.currentSelectRegisterPersonInfo?.physicalExamNumber ??
          //         '',
          //     'HEIECG',
          //   ),
          //   "createCheckup",
          //   1500,
          // );
        },
        label: const SizedBox(
          width: 240,
          height: 60,
          child: Center(
            child: Text(
              '提交',
              style: TextStyle(
                fontSize: 26,
                color: Colors.white,
              ),
            ),
          ),
        ),
      );
    });
  }

  Widget _buildContent() {
    return const TwelveHeartRate();
  }

  ///开启拖拽
  Future<void> onStartDrag() async {
    JSPlateForm.beginWindowDrag(_windowName);
  }

  ///结束拖拽
  Future<void> onEndDrag() async {
    JSPlateForm.endWindowDrag(_windowName);
  }

  ///最小化
  Future<void> onMinimize() async {
    JSPlateForm.minimizeWindow(_windowName);
  }

  ///最大化
  Future<void> onMaximize() async {
    JSPlateForm.maximizeWindow(_windowName);
  }

  ///关闭
  Future<void> onWindowClose() async {
    JSPlateForm.closeWindow(_windowName);
  }
}

class _PatientAddIconProxyWidget extends StatelessWidget {
  final Widget baseIcon;

  const _PatientAddIconProxyWidget({
    Key? key,
    required this.baseIcon,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Stack(
      children: [
        baseIcon,
        const Positioned(
          right: 0,
          top: 0,
          child: Icon(
            Icons.add,
            size: 16,
            color: Colors.white,
          ),
        )
      ],
    );
  }
}
