import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/process/workspace/rpc_bridge.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:image_picker/image_picker.dart';
import 'package:vitalapp/architecture/storage/storage.dart';
import 'package:vitalapp/rpc.dart';
import 'package:vitalapp/store/store.dart';

class MeasureDataHelper {
  /// 这边还没做压缩 缩略图
  static Future<bool?> saveImage(
    Uint8List imageBytes,
    String patientCode,
    String recordCode,
    String remedicalCode,
    String measuredData,
    VidImageSource source,
  ) async {
    try {
      XFile xFile = XFile.fromData(
        imageBytes,
        name: remedicalCode + '.jpg',
      );

      final measuredFileToken = await rpc.storage.upload(xFile);
      if (source == VidImageSource.RemoteUS) {
        return await rpc.remoteUltrasound.vitalAddRemedicalMeasuredInfoAsync(
          AddRemedicalMeasuredInfoRequest(
            recordCode: recordCode,
            businessType: BusinessTypeEnum.RemoteDiagnosis,
            remedicalMeasuredInfos: [
              AddRemedicalMeasuredInfoDTO(
                remedicalCode: remedicalCode,
                measuredFileToken: measuredFileToken,
                previewFileToken: measuredFileToken,
                measuredData: measuredData,
              )
            ],
            token: Store.user.token,
          ),
        );
      }
      final result = await rpc.recordInfo.addRemedicalMeasuredInfoAsync(
        AddRemedicalMeasuredInfoRequest(
          recordCode: recordCode,
          businessType: source == VidImageSource.Consultation
              ? BusinessTypeEnum.LiveConsultation
              : BusinessTypeEnum.RemoteDiagnosis,
          remedicalMeasuredInfos: [
            AddRemedicalMeasuredInfoDTO(
              remedicalCode: remedicalCode,
              measuredFileToken: measuredFileToken,
              previewFileToken: measuredFileToken,
              measuredData: measuredData,
            )
          ],
          token: Store.user.token,
        ),
      );
      return result;
    } catch (e) {
      logger.e("addRemedicalMeasuredInfoAsync exception:", e);
    }
    return null;
  }
}
