import 'dart:convert';

import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vitalapp/managers/interfaces/exam.dart';
import 'package:vitalapp/store/store.dart';

class TableInputDialogController extends GetxController {
  TableInputDialogController({
    required this.tableDataConfig,
  });
  final _examManager = Get.find<IExamManager>();

  final List<TableElementConfig> tableDataConfig;

  final List<TableElementData> tableData = [];

  Future<void> onConfirm(
    String? physicalExamNumber,
    String? keyValue,
  ) async {
    final result = <String, String>{};
    tableData.forEach((element) {
      String vlaue = element.textController.text;
      if (vlaue.isEmpty) {
        vlaue = '-';
      }
      result[element.config.id] = vlaue;
    });
    await createCheckup(physicalExamNumber, keyValue, result);
    // Get.back(result: TableInputResult(data: result));
  }

  ///  体检  检查提交
  Future<void> createCheckup(
    String? physicalExamNumber,
    String? keyValue,
    Map<String, dynamic> diagnosisDataValue,
  ) async {
    // Map<String, dynamic> input = diagnosisDataValue;
    // Map<String, dynamic> output = {};

    // input.forEach((key, value) {
    //   value.forEach((innerKey, innerValue) {
    //     output[innerKey] = innerValue;
    //   });
    // });
    var result = await _examManager.createExam(CreateExamRequest(
      key: keyValue ?? "HEIBasic",
      examData: jsonEncode(diagnosisDataValue),
      physicalExamNumber:
          Store.user.currentSelectRegisterPersonInfo?.physicalExamNumber,
    ));
    if (result == true) {
      Get.back();
    }
    print(result);
  }

  void focusPre() {
    int index = getCurrFocusIndex();
    if (index != -1) {
      if (index > 0) {
        moveFocus(index - 1);
      }
      if (index == 0) {
        moveFocus(tableData.length - 1);
      }
    }
  }

  void focusNext() {
    int index = getCurrFocusIndex();
    if (index != -1) {
      if (index < tableData.length - 1) {
        moveFocus(index + 1);
      }
      if (index == tableData.length - 1) {
        moveFocus(0);
      }
    }
  }

  int getCurrFocusIndex() {
    for (int i = 0; i < tableData.length; i++) {
      final FocusNode focusNode = tableData[i].focusNode;
      if (focusNode.hasFocus) {
        return i;
      }
    }
    return -1;
  }

  void moveFocus(int index) {
    print('moveFocus: $index');
    if (index >= 0 && index < tableData.length) {
      tableData[index].focusNode.requestFocus();
      // 如果该行有值，下一帧执行，选择全部
      if (tableData[index].textController.text.isNotEmpty) {
        WidgetsBinding.instance.addPostFrameCallback((timeStamp) {
          Future.delayed(Duration(milliseconds: 30), () {
            tableData[index].textController.selection = TextSelection(
              baseOffset: 0,
              extentOffset: tableData[index].textController.text.length,
            );
          });
        });
      }
    }
  }

  @override
  void onInit() {
    super.onInit();
    // 根据 tableDataConfig 初始化 tableData
    tableDataConfig.forEach((element) {
      tableData.add(TableElementData(element));
    });
  }

  @override
  void onReady() {
    super.onReady();
  }

  @override
  void onClose() {
    super.onClose();
  }
}

/// 表格配置项 (入参)
class TableElementConfig {
  final String id;
  final String name;
  final String unit;
  final String? initValue;
  TableElementConfig(
      {required this.id,
      required this.name,
      required this.unit,
      this.initValue});
}

/// 表格输入结果 (出参)
class TableInputResult {
  Map<String, String> data;
  TableInputResult({required this.data});
}

/// 表格元素数据（UI数据）
class TableElementData {
  final TableElementConfig config;
  final TextEditingController textController;
  FocusNode focusNode = FocusNode();
  TableElementData(this.config)
      : textController = TextEditingController(text: config.initValue);
}
