// ignore_for_file: must_be_immutable

import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vitalapp/architecture/utils/prompt_box.dart';
import 'package:vitalapp/components/table/table.dart';
import 'package:vitalapp/managers/interfaces/debouncer.dart';
import 'package:vitalapp/managers/interfaces/device.dart';
import 'package:vitalapp/managers/interfaces/patient.dart';
import 'package:vitalapp/pages/medical/widgets/health_check/health_check_list/controller.dart';
import 'package:vitalapp/pages/medical_checkup_station/registration/state/list.dart';

class HealthCheckTable extends GetView<HealthCheckListController> {
  HealthCheckTable({
    super.key,
    required this.onRowTap,
    required this.onCheckTap,
  }) {
    controller.onScanCode.addListener(_OnScanCode);
  }

  ValueCallback<ResidentModel> onRowTap;
  ValueCallback<ResidentModel> onCheckTap;
  final debouncer = Debouncer(milliseconds: 50);
  final _patientManager = Get.find<IPatientManager>();

  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        Expanded(
          child: GetBuilder<HealthCheckListController>(
            id: "healthCheck_table",
            builder: (_) {
              return VitalTable<ResidentModel>(
                autoHeight: false,
                noDataHintText: "暂无数据",
                columns: controller.buildTableColumns(onCheckTap),
                source: controller.residentList,
                loading: controller.tableLoading,
                onRowSelected: (value, index, idxs) {},
                onRowTap: (index) async {
                  ResidentModel resident = controller.residentList[index];
                  await controller.changeHeartCheck(resident);
                  onRowTap.call(resident);
                },
                currectSelected: controller.currectSelected,
                onAllRowsSelected: (value, idxs) => {},
                headerTextStyle: const TextStyle(
                  fontSize: 20,
                ),
              );
            },
          ),
        ),
        HealthCheckTablePagination(),
        SizedBox(
          height: 16,
        )
      ],
    );
  }

  void _OnScanCode(Object sender, String e) {
    print("扫码体检号：$e");
    debouncer.run(() async {
      if (controller.residentList
          .any((element) => element.physicalExamNumber == e)) {
        var data = controller.residentList
            .firstWhere((element) => element.physicalExamNumber == e);
        await controller.changeHeartCheck(data);
        onCheckTap.call(data);
      } else {
        RegisterPersonInfoDTO? registerPersonInfo = await _patientManager
            .getRegisterPersonInfoByPhysicalExamNumberAsync(
          physicalExamNumber: e,
        );
        if (registerPersonInfo == null) {
          PromptBox.toast("未找到此体检号");
          return;
        }
        String statusStr = '';
        ExamStateEnum status = registerPersonInfo.state;
        switch (status) {
          case ExamStateEnum.Unchecked:
            statusStr = "未体检";
            break;
          case ExamStateEnum.Invalid:
            statusStr = "已作废";
            break;
          case ExamStateEnum.Inspected:
            statusStr = "已体检";
            break;
          case ExamStateEnum.Reported:
            statusStr = "已报告";
            break;
        }
        var data = ResidentModel(
          name: registerPersonInfo.name ?? '',
          idNumber: registerPersonInfo.iDCardNo ?? '',
          code: registerPersonInfo.code ?? '',
          homeAddress: registerPersonInfo.adress,
          age: controller.getAgeOfIdNumber(registerPersonInfo.iDCardNo ?? ''),
          physicalExamNumber: registerPersonInfo.physicalExamNumber,
          phone: registerPersonInfo.phone,
          finishedExamKeys: registerPersonInfo.finishedExamKeys,
          physicalExamStatus: statusStr,
          birthDay: registerPersonInfo.birthday,
          sex: registerPersonInfo.sex,
        );
        await controller.changeHeartCheck(data);
        onCheckTap.call(data);
      }
    });
  }
}

class HealthCheckTablePagination extends GetView<HealthCheckListController> {
  const HealthCheckTablePagination({
    super.key,
  });
  @override
  Widget build(BuildContext context) {
    return GetBuilder<HealthCheckListController>(
      id: "healthCheck_table_pagination",
      builder: (_) {
        return Row(
          mainAxisAlignment: MainAxisAlignment.end,
          children: [
            FTablePagination(
              currListLength: controller.appointmentModelListLength,
              currPageIndex: controller.currPageIndex,
              pageSize: 10,
              onChangePage: (pageIndex, listLength) {
                controller.getRegisterInfoPage(
                  pageIndex: pageIndex,
                  pageSize: listLength,
                );
                // _tableScrollToTop();
              },
              totalItemNumText: i18nBook.common.ofTerm
                  .translate(['${controller.appointmentModelListLength}']),
              previousFivePageText: i18nBook.common.previousFivePageText.t,
              nextFivePageText: i18nBook.common.nextFivePageText.t,
            ),
            SizedBox(
              width: 20,
            )
          ],
        );
      },
    );
  }
}
