// ignore_for_file: must_be_immutable

import 'dart:convert';

import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vitalapp/components/no_data_view.dart';
import 'package:vitalapp/managers/interfaces/dictionary.dart';
import 'package:vitalapp/pages/medical/controller.dart';
import 'package:vitalapp/pages/medical/widgets/health_heart_check/health_check_left/index.dart';
import 'package:vitalapp/pages/medical/widgets/health_heart_check/health_check_list/controller.dart';
import 'package:vitalapp/pages/medical_checkup_station/registration/state/list.dart';
import 'package:vitalapp/pages/patient/detail/widgets/last_record.dart';

class HeartTableCheck extends StatefulWidget {
  HeartTableCheck({
    super.key,
    this.checkDialog,
    required this.checkKey,
  });

  final String checkKey;
  Widget? checkDialog;

  @override
  State<HeartTableCheck> createState() => _HeartTableCheckState();
}

class _HeartTableCheckState extends State<HeartTableCheck> {
  final controller = Get.find<HeartCheckListController>();
  final medicalController = Get.put(MedicalController());

  ResidentModel currentResident = ResidentModel(idNumber: "");

  @override
  void initState() {
    medicalController.onSelectExam.addListener(selectExamEvent);
    super.initState();
  }

  @override
  void dispose() {
    medicalController.onSelectExam.removeListener(selectExamEvent);
    Get.delete<MedicalController>();

    super.dispose();
  }

  void selectExamEvent(sender, e) async {
    if (e) {
      await _onResponseClick(currentResident, false);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      children: [
        Expanded(
          child: HeartCheckLeft(
            onRowTap: (value) async {
              await _onResponseClick(value, false);
            },
            onCheckTap: (value) async {
              /// 这是表格选中
              controller.currectSelected =
                  controller.residentList.indexOf(value);
              controller.update(["heartCheck_table"]);
              await _onResponseClick(value, true);
            },
          ),
        ),
        Container(
          width: 320,
          decoration: BoxDecoration(
            border: Border(
              left: BorderSide(
                color: Colors.grey[300]!,
                width: 1,
              ),
            ),
          ),
          child: _buildDetail(),
        )
      ],
    );
  }

  Future<void> _onResponseClick(ResidentModel value, bool isBtn) async {
    medicalController.diagnosisDataValue.clear();
    controller.tableData == null;
    controller.update(['heart_detail']);
    if (value.physicalExamNumber?.isEmpty ?? true) {
      return;
    }
    List<ExamDTO>? examList = await controller.registrationManager
        .getExamListByPhysicalExamNumberAsync(
            physicalExamNumber: value.physicalExamNumber ?? '');
    ExamDTO? currentExam =
        examList?.firstWhereOrNull((element) => element.key == widget.checkKey);

    if (isBtn) {
      Get.dialog(widget.checkDialog ?? Container());
    }
    if (currentExam == null) {
      controller.tableData = null;
    } else {
      controller.tableData = await getTableData(
        currentExam.examData ?? '',
      );
    }

    controller.update(['heart_detail']);

    // Get.dialog(checkDialog ?? Container());
  }

  Widget _buildDetail() {
    return GetBuilder<HeartCheckListController>(
      id: "heart_detail",
      builder: (_) {
        if (controller.tableData == null || controller.tableData?.length == 0)
          return VNoDataView();
        return Column(
          children: [
            LastRecordTable(
              columnNames: const ['检测项目', '检测结果', '单位'],
              tableData: controller.tableData ?? [[]],
            )
          ],
        );
      },
    );
  }

  Future<List<List<String>>> getTableData(String data) async {
    var currentDiagnosis = <List<String>>[];

    var jsonData = json.decode(data);
    List<String> keys = jsonData.keys.toList();

    List<DictionaryWithUnitDTO>? dtos = [];

    dtos = await Get.find<IDictionaryManager>()
            .getDictionaryNameAndUnitByKeysAsync(keys) ??
        [];

    for (var key in keys) {
      if (key == "ECG_POINT" || key == "ECG_POINT12") {
        continue;
      }
      var dto = dtos.firstWhereOrNull((item) => item.key == key);
      if (dto == null) {
        continue;
      }
      final value = jsonData[key].toString();
      if (value.isEmpty) {
        // 不展示空值
        continue;
      }
      currentDiagnosis.add([
        dto.name ?? '',
        value,
        dto.unit ?? '',
      ]);
    }

    return currentDiagnosis;
  }
}
