import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vitalapp/pages/medical/widgets/twelve_ecg_view/widgets/current_screen_ecg_data_dialog.dart';

import '../index.dart';

class FullScreenEcgDataDialog extends GetView<TwelveEcgViewController> {
  const FullScreenEcgDataDialog({Key? key}) : super(key: key);

  // 主视图
  Widget _buildView(BuildContext context) {
    const designWidth = 1280.0; // 设计尺寸宽度：1280
    final width = Get.width;
    final scale = width / designWidth; // 计算缩放比例
    final ScrollController scrollController = ScrollController();
    // 创建一个GlobalKey
    final GlobalKey containerKey = GlobalKey();

    return WillPopScope(
      onWillPop: () async {
        return false;
      },
      child: Container(
        width: Get.width * 0.9 / scale,
        height: 240 * 3,
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.circular(10),
        ),
        clipBehavior: Clip.antiAlias,
        child: Column(
          children: [
            _buildHead(),
            Expanded(
              child: Scrollbar(
                thumbVisibility: true,
                thickness: 10,
                radius: const Radius.circular(10),
                controller: scrollController,
                child: SingleChildScrollView(
                  controller: scrollController,
                  padding:
                      const EdgeInsets.only(left: 10, right: 10, bottom: 20),
                  physics: const BouncingScrollPhysics(),
                  scrollDirection: Axis.horizontal,
                  child: Container(
                    decoration: BoxDecoration(
                      border: Border.all(
                        color: Colors.grey.withOpacity(0.5),
                        width: 1,
                      ),
                    ),
                    key: containerKey,
                    clipBehavior: Clip.hardEdge,
                    child: GestureDetector(
                      onTapDown: (TapDownDetails details) {
                        final RenderBox referenceBox =
                            context.findRenderObject() as RenderBox;
                        final Offset localPosition =
                            referenceBox.globalToLocal(details.globalPosition);

                        // 使用GlobalKey来获取Container的RenderBox
                        final RenderBox renderBox = containerKey.currentContext
                            ?.findRenderObject() as RenderBox;

                        // 获取尺寸（如果renderBox不为null）
                        final size = renderBox.size;
                        final int areaIndex =
                            (localPosition.dy / size.height * 12).floor();
                        openCurrentScreenDialog(areaIndex - 1);
                      },
                      child: CustomPaint(
                        foregroundPainter: EcgPainterForAll(
                          allPoints: controller.allPoints,
                          yMaxList: controller.yMaxList,
                        ),
                        painter: GridBackgroundPainterForAll(),
                        size: const Size(3000, 200),
                      ),
                    ),
                  ),
                ),
              ),
            ),
          ],
        ),
      ),
    );
  }

  /// 打开单个心电图弹窗
  void openCurrentScreenDialog(int currentIndex) {
    Get.dialog(
      CurrentScreenEcgDataDialog(currentIndex: currentIndex),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Dialog(
      child: _buildView(context),
    );
  }

  /// 构建弹窗顶部，右侧显示关闭按钮
  Widget _buildHead() {
    return SizedBox(
      height: 50,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.end,
        children: [
          IconButton(
            onPressed: () => Get.back(),
            icon: const Icon(
              Icons.close,
              size: 30,
            ),
          ),
        ],
      ),
    );
  }
}
