// ignore_for_file: must_be_immutable

import 'package:date_format/date_format.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vitalapp/components/button.dart';
import 'package:vitalapp/components/dialog_input.dart';
import 'package:vitalapp/components/dialog_time.dart';
import 'package:vitalapp/components/input.dart';
import 'package:vitalapp/pages/medical_checkup_station/appointment/controller.dart';
import 'package:vitalapp/pages/medical_checkup_station/appointment/state/list.dart';

class AppointmentFormDialog extends GetView<AppointmentController> {
  AppointmentFormDialog({
    Key? key,
    this.appointment,
    this.isEdit = false,
  }) : super(key: key);
  AppointmentModel? appointment;

  bool? isEdit;
  // 主视图
  Widget _buildView() {
    return Column(
      children: [
        _buildHead(),
        _buildInputItem(
          "体检名称",
          value: appointment?.appointmentName ?? '',
          maxLines: 1,
          onChanged: (v) {
            appointment?.appointmentName = v;
            //controller.update(["appointment_Form"]);
          },
        ),
        _buildInputItem(
          "体检地点",
          value: appointment?.appointmentAddress ?? '',
          maxLines: 1,
          onChanged: (v) {
            appointment?.appointmentAddress = v;
          },
        ),
        _buildInputItem(
          "体检开始时间",
          value: _getTime(appointment?.appointmentStartTime),
          onTap: _onChangeStartTime,
          suffixIcon: Icon(
            Icons.chevron_right_rounded,
            size: 30,
          ),
        ),
        _buildInputItem(
          "体检结束时间",
          value: _getTime(appointment?.appointmentEndTime),
          onTap: _onChangeEndTime,
          suffixIcon: Icon(
            Icons.chevron_right_rounded,
            size: 30,
          ),
        ),
        _buildInputItem(
          "体检对象",
          maxLines: 1,
          value: appointment?.appointmentObject,
          onChanged: (v) {
            appointment?.appointmentObject = v;
          },
        ),
        _buildInputItem(
          "体检说明",
          maxLines: 5,
          value: appointment?.appointmentDescription,
          onChanged: (v) {
            appointment?.appointmentDescription = v;
          },
        ),
        SizedBox(
          height: 10,
        ),
        _buildCreate(),
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    return GetBuilder<AppointmentController>(
      id: "appointment_Form",
      builder: (_) {
        return Dialog(
          child: Container(
            width: 900,
            height: 650,
            child: _buildView(),
          ),
        );
      },
    );
  }

  /// 构建创建按钮
  Widget _buildCreate() {
    return VButton(
      child: isEdit! ? Text("更新") : Text("创建"),
      onTap: () {
        controller.formController.saveHealthExamBooking(
          appointment!,
          isEdit: isEdit!,
        );
      },
    );
  }

  /// 构建弹窗顶部，右侧显示关闭按钮
  Widget _buildHead() {
    return SizedBox(
      height: 60,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Container(
            padding: const EdgeInsets.only(left: 16),
            child: Text(
              isEdit! ? "编辑预约模板" : "新建预约模板",
              style: TextStyle(fontSize: 24, fontWeight: FontWeight.bold),
            ),
          ),
          IconButton(
            onPressed: () => Get.back(),
            icon: const Icon(
              Icons.close,
              size: 30,
            ),
          ),
        ],
      ),
    );
  }

  /// 构建表格输入框
  Widget _buildInputItem(
    String title, {
    String? value,
    Function()? onTap,
    Widget? suffixIcon,
    int? maxLines,
    void Function(String)? onChanged,
    TextEditingController? controller,
  }) {
    return Container(
      padding: const EdgeInsets.only(
        left: 80,
        right: 110,
      ),
      child: Row(
        children: [
          Container(
            width: 150,
            padding: const EdgeInsets.symmetric(vertical: 16),
            child: Text(
              title,
              style: const TextStyle(fontSize: 22),
            ),
          ),
          Expanded(
            child: VInput(
              initialValue: value,
              radius: 4,
              readOnly: onTap == null ? false : true,
              suffixIcon: suffixIcon,
              maxLines: maxLines,
              onChanged: onChanged,
              controller: controller,
              onTap: () {
                onTap?.call();
              },
            ),
          )
        ],
      ),
    );
  }

  void _onChangeName() async {
    var result = await VDialogInput(
      initialValue: appointment?.appointmentName ?? '',
    ).show();
    if (result != null) {
      appointment?.appointmentName = result;
      controller.update(["appointment_Form"]);
    }
  }

  void _onChangeAddress() async {
    var result = await VDialogInput(
      initialValue: appointment?.appointmentAddress ?? '',
    ).show();
    if (result != null) {
      appointment?.appointmentAddress = result;
      controller.update(["appointment_Form"]);
    }
  }

  void _onChangeStartTime() async {
    DateTime now = DateTime.now();
    DateTime? result;
    var initialValue = appointment?.appointmentStartTime ??
        now.add(
          Duration(
            minutes: 1,
          ),
        );
    if (kIsWeb) {
      result = await showDatePicker(
        context: Get.context!,
        initialDate: initialValue,
        firstDate: DateTime(2000),
        lastDate: DateTime(2100),
      );
      if (result == null) {
        return;
      }
      var time = await showTimePicker(
        context: Get.context!,
        initialTime: TimeOfDay(hour: 9, minute: 0),
        initialEntryMode: TimePickerEntryMode.input,
      );
      if (time != null && result != null) {
        result = result.add(Duration(hours: time.hour, minutes: time.minute));
      }
    } else {
      result = await VDialogDateTime(
        minimumDate: now,
        initialValue: initialValue,
        // maxValue: appointment?.appointmentEndTime,
      ).show();
    }
    if (result != null) {
      appointment?.appointmentStartTime = result;
      appointment?.appointmentEndTime = null;
      controller.update(["appointment_Form"]);
    }
  }

  void _onChangeEndTime() async {
    DateTime now = DateTime.now();
    DateTime latestTime = now;
    DateTime? result;
    var initialValue = appointment?.appointmentEndTime ??
        appointment?.appointmentStartTime!.add(
          Duration(
            minutes: 1,
          ),
        ) ??
        DateTime.now();
    if (appointment?.appointmentStartTime != null) {
      latestTime = DateTime(
        appointment!.appointmentStartTime!.year,
        appointment!.appointmentStartTime!.month,
        appointment!.appointmentStartTime!.day,
        23,
        59,
        59,
      );
    }
    if (kIsWeb) {
      result = await showDatePicker(
        context: Get.context!,
        initialDate: initialValue,
        firstDate: DateTime(2000),
        lastDate: DateTime(2100),
      );
      if (result == null) {
        return;
      }
      var time = await showTimePicker(
        context: Get.context!,
        initialTime: TimeOfDay(hour: 9, minute: 0),
        initialEntryMode: TimePickerEntryMode.input,
      );
      if (time != null && result != null) {
        result = result.add(Duration(hours: time.hour, minutes: time.minute));
      }
    } else {
      result = await VDialogDateTime(
        minimumDate: appointment?.appointmentStartTime,
        initialValue: initialValue,
        maxValue: latestTime,
      ).show();
    }
    if (result != null) {
      appointment?.appointmentEndTime = result;
      controller.update(["appointment_Form"]);
    }
  }

  void _onChangeObject() async {
    var result = await VDialogInput(
      initialValue: appointment?.appointmentObject ?? '',
    ).show();
    if (result != null) {
      appointment?.appointmentObject = result;
      controller.update(["appointment_Form"]);
    }
  }

  void _onChangeDescription() async {
    var result = await VDialogInput(
      initialValue: appointment?.appointmentDescription ?? '',
      maxLines: 4,
      inputHeight: 80,
    ).show();
    if (result != null) {
      appointment?.appointmentDescription = result;
      controller.update(["appointment_Form"]);
    }
  }

  String _getTime(DateTime? currentTime) {
    if (currentTime == null) {
      return "";
    }
    return formatDate(currentTime, [yyyy, "-", mm, "-", dd, " ", HH, ":", nn]);
  }
}
