import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:vitalapp/components/dialog_time.dart';
import 'package:vitalapp/components/input.dart';

class FilterTime extends StatelessWidget {
  final _startController = TextEditingController();
  final _endController = TextEditingController();
  final void Function(DateTime, DateTime) onConfirm;
  DateTime _startTime = DateTime.now();
  DateTime _endTime = DateTime.now();

  FilterTime({
    required this.onConfirm,
    super.key,
  });

  @override
  Widget build(BuildContext context) {
    var now = DateTime.now();
    DateTime newDay = DateTime(now.year, now.month, now.day);
    _startTime = newDay.add(Duration(days: -30));
    _endTime = newDay.add(Duration(hours: 23, minutes: 59));
    // 设置日期格式为 年-月-日 时:分
    String startFormattedDate =
        DateFormat('yyyy-MM-dd HH:mm').format(_startTime);
    String endFormattedDate = DateFormat('yyyy-MM-dd HH:mm').format(_endTime);
    _startController.text = startFormattedDate;
    _endController.text = endFormattedDate;
    return Center(
      child: Container(
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: BorderRadius.circular(8.0), // 设置圆角的半径
        ),
        width: 500,
        height: 300,
        child: Column(
          children: [
            SizedBox(
              height: 30,
            ),
            _buildInputItem(
              "开始时间",
              value: startFormattedDate,
              onTap: _onChangeStartTime,
              suffixIcon: Icon(
                Icons.chevron_right_rounded,
                size: 30,
              ),
              controller: _startController,
            ),
            SizedBox(
              height: 30,
            ),
            _buildInputItem(
              "结束时间",
              value: endFormattedDate,
              onTap: _onChangeEndTime,
              suffixIcon: Icon(
                Icons.chevron_right_rounded,
                size: 30,
              ),
              controller: _endController,
            ),
            Expanded(child: SizedBox()),
            ElevatedButton(
              onPressed: () {
                onConfirm.call(_startTime, _endTime);
                Get.back();
              },
              style: ButtonStyle(
                backgroundColor:
                    MaterialStatePropertyAll(Theme.of(context).primaryColor),
                elevation: const MaterialStatePropertyAll(0),
                padding: MaterialStatePropertyAll(
                  EdgeInsets.symmetric(horizontal: 16, vertical: 8),
                ),
                alignment: Alignment.center,
                shape: MaterialStatePropertyAll(
                  RoundedRectangleBorder(
                    borderRadius: BorderRadius.circular(16),
                  ),
                ),
              ),
              child: Text(
                "确定",
                style: TextStyle(
                  color: Colors.white,
                  fontSize: 18,
                ),
              ),
            ),
            SizedBox(
              height: 20,
            ),
          ],
        ),
      ),
    );
  }

  /// 构建表格输入框
  Widget _buildInputItem(
    String title, {
    String? value,
    Function()? onTap,
    Widget? suffixIcon,
    int? maxLines,
    void Function(String)? onChanged,
    TextEditingController? controller,
  }) {
    return Container(
      padding: const EdgeInsets.only(
        left: 20,
        right: 20,
      ),
      child: Row(
        children: [
          Container(
            width: 150,
            padding: const EdgeInsets.symmetric(vertical: 16),
            child: Text(
              title,
              style: const TextStyle(fontSize: 22),
            ),
          ),
          Expanded(
            child: VInput(
              initialValue: value,
              radius: 4,
              readOnly: onTap == null ? false : true,
              suffixIcon: suffixIcon,
              maxLines: maxLines,
              onChanged: onChanged,
              controller: controller,
              onTap: () {
                onTap?.call();
              },
            ),
          )
        ],
      ),
    );
  }

  void _onChangeStartTime() async {
    DateTime now = DateTime.now();
    DateTime? result;
    var initialValue = _startTime;
    if (kIsWeb) {
      result = await showDatePicker(
        context: Get.context!,
        initialDate: initialValue,
        firstDate: DateTime(2000),
        lastDate: DateTime(2100),
      );
      if (result == null) {
        return;
      }
      var time = await showTimePicker(
        context: Get.context!,
        initialTime: TimeOfDay(hour: 9, minute: 0),
        initialEntryMode: TimePickerEntryMode.input,
      );
      if (time != null) {
        result = result.add(Duration(hours: time.hour, minutes: time.minute));
      }
    } else {
      result = await VDialogDateTime(
        // minimumDate: now,
        initialValue: initialValue,
      ).show();
    }
    if (result != null) {
      String startFormattedDate = DateFormat('yyyy-MM-dd HH:mm').format(result);
      _startTime = result;
      _startController.text = startFormattedDate;
    }
  }

  Future<void> _onChangeEndTime() async {
    DateTime now = DateTime.now();
    DateTime? result;
    var initialValue = _endTime;
    if (kIsWeb) {
      result = await showDatePicker(
        context: Get.context!,
        initialDate: initialValue,
        firstDate: DateTime(2000),
        lastDate: DateTime(2100),
      );
      if (result == null) {
        return;
      }
      var time = await showTimePicker(
        context: Get.context!,
        initialTime: TimeOfDay(hour: 9, minute: 0),
        initialEntryMode: TimePickerEntryMode.input,
      );
      if (time != null) {
        result = result.add(Duration(hours: time.hour, minutes: time.minute));
      }
    } else {
      result = await VDialogDateTime(
        minimumDate: now,
        initialValue: initialValue,
      ).show();
    }
    if (result != null) {
      _endTime = result;
      String endFormattedDate = DateFormat('yyyy-MM-dd HH:mm').format(result);
      _endController.text = endFormattedDate;
    }
  }
}
