import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vitalapp/architecture/utils/prompt_box.dart';
import 'package:vitalapp/architecture/utils/verify_permissions.dart';
import 'package:vitalapp/pages/id_card_scan/controller.dart';
import 'package:vitalapp/pages/id_card_scan/view.dart';
import 'package:vitalapp/pages/medical_checkup_station/registration/controller.dart';
import 'package:vitalapp/pages/medical_checkup_station/registration/widgets/card_reader/view.dart';
import 'package:vitalapp/pages/medical_checkup_station/registration/widgets/floating_action_button/action_button.dart';
import 'package:vitalapp/pages/medical_checkup_station/registration/widgets/floating_action_button/expandable_fab.dart';
import 'package:vitalapp/pages/medical_checkup_station/registration/widgets/form/index.dart';

/// 【弃用，可能后面还需要】
class RegistrationActionButton extends GetView<RegistrationController> {
  const RegistrationActionButton({super.key});

  /// 读卡逻辑
  void onReadCardClicked() async {
    PatientDTO? result;

    result = await Get.dialog<PatientDTO>(
      const RegistrationCardReaderDialog(),
      barrierDismissible: false,
    );
    if (result != null) {
      controller.formController.confirmSubmit(
        patient: result,
        isVital: false,
      );
    }
  }

  /// 扫码识别逻辑
  void onIdCardScanClicked() async {
    var verifyResult = await VerifyPermissions.verifyCameraPermissions();
    if (!verifyResult) {
      return;
    }
    final IdCardScanResult? result = await Get.to<IdCardScanResult>(
      () => const IdCardScanPage(),
    );
    if (result != null && result.success) {
      PromptBox.toast("身份证信息识别成功");
      PatientDTO patientInfo = result.patientBaseDTO as PatientDTO;
      controller.formController.confirmSubmit(
        patient: patientInfo,
        isVital: false,
      );
    } else {
      print("识别取消");
    }
  }

  /// 手动输入信息逻辑
  void onManualInputClicked() async {
    controller.patientInfo = PatientDTO();
    final PatientDTO? result = await Get.dialog<PatientDTO>(
      RegistrationFormDialog(patient: controller.patientInfo),
      barrierDismissible: false,
    );
    controller.formController.createResidentInList(
      patient: result!,
      isVital: false,
    );
  }

  void _showAction(BuildContext context, int index) {
    if (index == 0) {
      onReadCardClicked();
      return;
    }
    if (index == 1) {
      onIdCardScanClicked();
      return;
    }
    if (index == 2) {
      onManualInputClicked();
      return;
    }
  }

  @override
  Widget build(BuildContext context) {
    return ExpandableFab(
      distance: 160,
      children: [
        ActionButton(
          onPressed: () => _showAction(context, 0),
          icon: const Icon(
            Icons.chrome_reader_mode,
            size: kIsWeb ? 26 : 36,
          ),
        ),
        ActionButton(
          onPressed: () => _showAction(context, 1),
          icon: const Icon(
            Icons.perm_contact_cal_rounded,
            size: kIsWeb ? 26 : 36,
          ),
        ),
        ActionButton(
          onPressed: () {
            _showAction(context, 2);
          },
          icon: const Icon(
            Icons.edit_document,
            size: kIsWeb ? 26 : 36,
          ),
        ),
      ],
    );
  }
}
