// ignore_for_file: must_be_immutable

import 'package:date_format/date_format.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vitalapp/architecture/utils/prompt_box.dart';
import 'package:vitalapp/components/button.dart';
import 'package:vitalapp/components/dialog_date.dart';
import 'package:vitalapp/components/dialog_input.dart';
import 'package:vitalapp/components/dialog_select.dart';
import 'package:vitalapp/components/input.dart';
import 'package:vitalapp/helper/id_card_helper.dart';
import 'package:vitalapp/pages/medical_checkup_station/registration/controller.dart';
import 'package:vitalapp/pages/medical_checkup_station/registration/state/list.dart';

class RegistrationFormDialog extends StatefulWidget {
  final PatientDTO? patient;
  final bool? isEdit;

  RegistrationFormDialog({Key? key, this.patient, this.isEdit = false})
      : super(key: key);

  @override
  _RegistrationFormDialogState createState() => _RegistrationFormDialogState();
}

class _RegistrationFormDialogState extends State<RegistrationFormDialog> {
  var controller = Get.find<RegistrationController>();
  @override
  Widget build(BuildContext context) {
    return GetBuilder<RegistrationController>(
      id: "registration_Form",
      builder: (_) {
        return Dialog(
          child: Container(
            width: 900,
            height: 580,
            decoration: BoxDecoration(
              color: Colors.white,
              borderRadius: BorderRadius.circular(8),
            ),
            child: _buildView(),
          ),
        );
      },
    );
  }

  Widget _buildView() {
    return Column(
      children: [
        _buildHead(),
        _buildInputItem(
          "身份证号",
          value: widget.patient?.cardNo ?? '',
          onTap: () async {
            var result = await VDialogInput(
              initialValue: widget.patient?.cardNo ?? '',
              title: "身份证号",
            ).show();
            if (result != null) {
              widget.patient?.cardNo = result;
              widget.patient?.birthday = _getBirthDayofIdNumber(result);
              controller.update(["registration_Form"]);
            }
          },
        ),
        _buildInputItem(
          "姓名",
          value: widget.patient?.patientName ?? '',
          onTap: () async {
            var result = await VDialogInput(
              initialValue: widget.patient?.patientName ?? '',
              title: "姓名",
            ).show();
            if (result != null) {
              widget.patient?.patientName = result;
              controller.update(["registration_Form"]);
            }
          },
        ),
        _buildInputItem(
          "性别",
          value: controller.formController.selectGenderList
              .firstWhereOrNull(
                  (element) => element.code == widget.patient?.patientGender)
              ?.name,
          onTap: () async {
            String? result =
                await VDialogSelect<VSelectGenderEnumModel, String>(
              source: controller.formController.selectGenderList,
              labelGetter: (data) => data.name,
              valueGetter: (data) => data.code.index.toString(),
              initialValue: widget.patient?.patientGender.index.toString(),
              title: "性别",
            ).show();
            if (result != null) {
              widget.patient?.patientGender =
                  GenderEnum.values[int.parse(result)];
              controller.update(["registration_Form"]);
            }
          },
          suffixIcon: Icon(
            Icons.chevron_right_rounded,
            size: 30,
          ),
        ),
        _buildInputItem(
          "生日",
          value: getBirthDay(widget.patient?.birthday),
          onTap: () async {
            var result = await VDialogDate(
              initialValue: widget.patient?.birthday,
              title: "生日",
              // maxValue: appointment?.appointmentEndTime,
            ).show();
            if (result != null) {
              widget.patient?.birthday = result;
              controller.update(["registration_Form"]);
            }
          },
          suffixIcon: Icon(
            Icons.chevron_right_rounded,
            size: 30,
          ),
        ),
        _buildInputItem(
          "手机号",
          value: widget.patient?.phone,
          onTap: () async {
            var result = await VDialogInput(
              initialValue: widget.patient?.phone ?? '',
              title: "手机号",
            ).show();
            if (result != null) {
              widget.patient?.phone = result;
              controller.update(["registration_Form"]);
            }
          },
        ),
        _buildInputItem(
          "家庭住址",
          value: widget.patient?.patientAddress ?? '',
          onTap: () async {
            var result = await VDialogInput(
              initialValue: widget.patient?.patientAddress ?? '',
              title: "家庭住址",
            ).show();
            if (result != null) {
              widget.patient?.patientAddress = result;
              controller.update(["registration_Form"]);
            }
          },
        ),
        _buildIdentityVerifyButton(),
      ],
    );
  }

  Widget _buildHead() {
    return SizedBox(
      height: 60,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Container(
            padding: const EdgeInsets.only(left: 16),
            child: Text(
              widget.isEdit! ? "编辑登记信息" : "体检信息录入",
              style: TextStyle(fontSize: 24, fontWeight: FontWeight.bold),
            ),
          ),
          IconButton(
            onPressed: () => Get.back(),
            icon: const Icon(
              Icons.close,
              size: 30,
            ),
          ),
        ],
      ),
    );
  }

  Widget _buildInputItem(String title,
      {String? value, Function()? onTap, Widget? suffixIcon, int? maxLines}) {
    return Container(
      padding: const EdgeInsets.only(
        left: 80,
        right: 110,
      ),
      child: Row(
        children: [
          Container(
            width: 150,
            padding: const EdgeInsets.symmetric(vertical: 16),
            child: Text(
              title,
              style: const TextStyle(fontSize: 22),
            ),
          ),
          Expanded(
            child: VInput(
              initialValue: value,
              radius: 4,
              readOnly: true,
              suffixIcon: suffixIcon,
              maxLines: maxLines,
              onTap: () {
                onTap?.call();
              },
            ),
          )
        ],
      ),
    );
  }

  Widget _buildIdentityVerifyButton() {
    return Container(
      padding: const EdgeInsets.symmetric(
        vertical: 12,
      ),
      child: VButton(
        label: "确定",
        onTap: () async {
          String? isValidate = await _validateForm(widget.patient);
          if (isValidate != null) {
            PromptBox.toast(isValidate);
            return;
          } else
            Get.back(result: widget.patient);
        },
      ),
    );
  }

  Future<String?> _validateForm(PatientDTO? patient) async {
    if (patient == null) {
      return "当前不存在病人";
    }
    if (patient.patientName == null || patient.patientName!.isEmpty) {
      return "请填写姓名";
    }
    if (patient.cardNo == null || patient.cardNo!.isEmpty) {
      return "请填写身份证";
    }

    bool isNotIDCard = IdCardHelper.validateIDCard(patient.cardNo ?? '');
    if (!isNotIDCard) {
      return "请填写正确的身份证号";
    }

    /// TODO 需求变更暂时删除
    // final selectedNormalCodes = crowdLabelsController.state.selectedNormalCodes;

    // if (selectedNormalCodes.length > 1) {
    //   return "人群分类：一般人群、儿童、孕妇、老年人，只可选择其一！";
    // }
    // final crowdLabelCodes = crowdLabelsController.state.selectedCodes;
    // if (crowdLabelCodes.isEmpty) {
    //   return "请选择人群分类";
    // }
    // if (state.gender == GenderEnum.Male &&
    //     crowdLabelCodes.contains('RQFL_YF')) {
    //   return "当前居民性别为“男”，人群分类不可选择孕妇！";
    // }
    return null;
  }

  DateTime? _getBirthDayofIdNumber(String idCardNumber) {
    if (idCardNumber.isEmpty) return DateTime(1970, 1, 1);
    final idCardRegex = RegExp(r'^(\d{6})(\d{4})(\d{2})(\d{2})(\d{3})(\d|X)$');
    final match = idCardRegex.firstMatch(idCardNumber);
    if (match != null) {
      final year = int.parse(match.group(2)!);
      final month = int.parse(match.group(3)!);
      final day = int.parse(match.group(4)!);

      final birthDate = DateTime(year, month, day);

      return birthDate;
    }
    return null; // 返回一个默认值，表示无法提取出生日期
  }

  String getBirthDay(DateTime? birthDay) {
    if (birthDay == null) {
      return "";
    }
    return formatDate(birthDay, [yyyy, "-", mm, "-", dd]);
  }
}
