import 'dart:convert';
import 'dart:ui' as ui;
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_printer_plus/flutter_printer_plus.dart'
    if (dart.library.html) "package:vitalapp/pages/medical_checkup_station/usb_print/web/flutter_printer_plus.dart";
import 'package:get/get.dart';
import 'package:print_image_generate_tool/print_image_generate_tool.dart';
import 'package:vitalapp/components/button.dart';
import 'package:vitalapp/pages/medical_checkup_station/usb_print/module/printer_info.dart';
import 'package:vitalapp/rpc.dart' as rpc;
import 'package:flutter/src/widgets/image.dart' as imgui;

class PrintPreview extends StatelessWidget {
  final List<Uint8List> imageList;
  final PrinterInfo printInfo;
  const PrintPreview(
      {super.key, required this.imageList, required this.printInfo});

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('标签打印'),
        actions: [
          IconButton(
            onPressed: () {
              Get.back();
            },
            icon: Icon(Icons.close),
          ),
        ],
        leading: Container(),
        leadingWidth: 0,
        surfaceTintColor: Colors.transparent,
      ),
      body: Container(
        alignment: Alignment.center,
        child: Padding(
          padding: const EdgeInsets.all(8.0),
          child: GridView.builder(
            gridDelegate: SliverGridDelegateWithFixedCrossAxisCount(
              crossAxisCount: 3, // 每行显示三个图片
              crossAxisSpacing: 8.0, // 水平间距
              childAspectRatio: 360 / 240,
            ),
            itemCount: imageList.length,
            itemBuilder: (context, index) {
              return FittedBox(
                fit: BoxFit.scaleDown, // 缩放方式，根据需要调整
                child: imgui.Image.memory(
                  imageList[index],
                  width: 360,
                  height: 240,
                ),
              );
            },
          ),
        ),
      ),
      bottomNavigationBar: BottomAppBar(
        color: Colors.transparent,
        surfaceTintColor: Colors.transparent,
        child: Container(
          height: 50.0,
          alignment: Alignment.center,
          child: SizedBox(
            width: 120,
            child: VButton(
              onTap: () async {
                if (kIsWeb) {
                  await printLabelWeb();
                } else {
                  await printLabelAndroid();
                }
              },
              child: Text('打印'),
            ),
          ),
        ),
      ),
    );
  }

  Future<void> printLabelWeb() async {
    List<String> labels = [];
    for (var data in imageList) {
      String base64Image = base64Encode(data);
      labels.add(base64Image);
    }
    if (labels.length > 0) {
      await rpc.rpc.platform.printLabels(labels);
    }
  }

  Future<void> printLabelAndroid() async {
    for (var data in imageList) {
      Uint8List rawRgbaBytes = await decodePng(data);
      await _androidPrintLabel(rawRgbaBytes);
      await Future.delayed(Duration(seconds: 1));
    }
  }

  Future<Uint8List> decodePng(Uint8List pngBytes) async {
    ui.Codec codec = await ui.instantiateImageCodec(pngBytes);
    ui.FrameInfo frameInfo = await codec.getNextFrame();

    Uint8List rawRgbaBytes = await frameInfo.image
        .toByteData(format: ui.ImageByteFormat.rawRgba)
        .then((byteData) => byteData!.buffer.asUint8List());
    codec.dispose();
    return rawRgbaBytes;
  }

  // //打印图层生成成功
  Future<void> _androidPrintLabel(Uint8List imageBytes) async {
    //指定的打印机
    final printerInfo = printInfo;
    //打印票据类型（标签、小票）
    final printTypeEnum = PrintTypeEnum.label;

    //也可以使用 ImageByteFormat.png
    final argbWidth = 360;
    final argbHeight = 240;
    //argbWidthPx、argbHeightPx 不要传值，默认为空就行
    var printData = await PrinterCommandTool.generatePrintCmd(
      imgData: imageBytes,
      printType: printTypeEnum,
      argbWidthPx: argbWidth,
      argbHeightPx: argbHeight,
    );
    if (printerInfo.isUsbPrinter) {
      // usb 打印
      final conn = UsbConn(printerInfo.usbDevice!);
      conn.writeMultiBytes(printData, 1024 * 3);
    } else if (printerInfo.isNetPrinter) {
      // 网络 打印
      final conn = NetConn(printerInfo.ip!);
      conn.writeMultiBytes(printData);
    }
  }
}
