import 'dart:typed_data';

import 'package:print_image_generate_tool/print_image_generate_tool.dart';

abstract class PrinterCommandTool {
  /// 生成打印圖片指令
  /// singleByteSizeLimit: 单次写入的长度限制
  /// 只要 imgData 不是使用 ImageByteFormat.rawRgba 格式转换的 unit8List
  /// argbWidthPx、argbHeightPx 不要传值，默认为空就行
  static Future<List<List<int>>> generatePrintCmd({
    String? imgPath,
    Uint8List? imgData,
    required PrintTypeEnum printType,
    int imgSizeLimit = 550 * 1000,
    int? argbWidthPx,
    int? argbHeightPx,
  }) async {
    return [];
  }

  /// 开启钱箱
  static List<int> generateEscCashDrawerCmd() {
    return [];
  }
}

/// 提供打印机搜索能力
abstract class FlutterPrinterFinder {
  /// 搜索 已连接(Android)/已安装驱动的(Windows) USB 打印机
  static Future<List<dynamic>> queryUsbPrinter() async {
    return [];
  }

  /// 搜索可用的网络打印机设备
  static Future<List<String>> queryPrinterIp() async {
    return [];
  }
}

class UsbConn extends PrintBaseConn {
  final usbDevice;

  UsbConn(this.usbDevice);
  Future<int> writeMultiBytes(
    List<List<int>> data,
    int singleWriteLimit,
  ) async {
    int writeCount = 0;
    for (int index = 0; index < data.length; index++) {
      final itemBytes = data[index];
      final resultCount = await writeBytes(itemBytes, singleWriteLimit);
      if (resultCount <= 0) {
        throw Exception('Print transmission interrupted');
      }
      writeCount += resultCount;
    }
    return writeCount;
  }

  // 写入数据
  Future<int> writeBytes(
    List<int> data,
    int singleWriteLimit,
  ) async {
    throw Exception('The platform is not supported temporarily');
  }

  // android 平台写入
  Future<int> writeBytesWithAndroid(
    List<int> bytes,
    int singleWriteLimit,
  ) {
    throw Exception('The platform is not supported temporarily');
  }

  // windows 平台写入
  Future<int> writeBytesWithWindows(
    List<int> bytes,
  ) {
    throw Exception('The platform is not supported temporarily');
  }
}

class NetConn extends PrintBaseConn {
  final String address;
  bool _isConnect = false;
  int? _sourcePort;

  NetConn(this.address);

  bool get connected => _isConnect;

  Future<bool> reset() async {
    try {
      await disconnect();
      await connect(throwError: true);
      return _isConnect;
    } catch (e) {
      rethrow;
    }
  }

  void _close() {
    throw Exception('The platform is not supported temporarily');
  }

  Future<bool> connect({
    int port = 9100,
    Duration? timeout = const Duration(seconds: 5),
    bool canRetry = true,
    bool throwError = false,
  }) async {
    throw Exception('The platform is not supported temporarily');
  }

  Future<bool> disconnect() async {
    throw Exception('The platform is not supported temporarily');
  }

  Future<int> writeMultiBytes(
    List<List<int>> data, {
    bool isDisconnect = true,
  }) async {
    int writeCount = 0;
    for (int index = 0; index < data.length; index++) {
      final itemBytes = data[index];
      final resultCount = await writeBytes(
        itemBytes,
        isDisconnect: isDisconnect,
      );
      if (resultCount <= 0) {
        throw Exception('Print transmission interrupted');
      }
      writeCount += resultCount;
    }
    return writeCount;
  }

  // 写入数据
  Future<int> writeBytes(
    List<int> data, {
    bool isDisconnect = true,
  }) async {
    throw Exception('The platform is not supported temporarily');
  }
}

abstract class PrintBaseConn {}
