import 'package:get/get.dart';

typedef RxModelProxyPropUpdater<T> = void Function(T x);

extension RxModelExtension<T> on Rx<T> {
  void updateProp(RxModelProxyPropUpdater<T> updater) {
    updater.call(value); // 更新字段
    refresh();
  }
}

class RxModelProxy2<T> extends Rx<T> {
  RxModelProxy2(super.initial);

  void updateProp(RxModelProxyPropUpdater<T> updater) {
    updater.call(value); // 更新字段
    refresh();
  }
}

class RxModelProxy<T> {
  RxModelProxy(T initialVal) {
    _value = Rx<T>(initialVal);
  }

  late final Rx<T> _value;

  void updateProp(RxModelProxyPropUpdater<T> updater) {
    final model = _value.value;
    updater.call(model); // 更新字段
    _value.refresh();
  }
}

class MyModelProxy extends RxModelProxy<MyModel> {
  MyModelProxy(super.initialVal);

  final dd = RxModelProxy<MyModel>(MyModel());
  final dl = Rx<MyModel>(MyModel());

  void test() {
    dl.updateProp((x) => x.name = "2");
    updateProp((x) => x.name = "111");
  }
}

class MyModel {
  String? name;
}
