import 'dart:convert';

import 'package:vitalapp/components/dialog_profile_time_input.dart';

class PatientDetailInfoModel {
  PatientDetailInfoModel();

  ///详细信息
  String? workUnit; //工作单位
  String? contactName; //联系人姓名
  String? contactPhone; //联系人电话
  String? permanentlyResideType; //常住类型
  String? bloodType; //血型
  String? rh; //RH
  String? educationLevel; //文化程度
  String? career; //职业
  String? maritalStatus; //婚姻状况
  String? providerPayments; //医疗费用支付方式
  String? providerPaymentsOther; //医疗费用支付方式

  factory PatientDetailInfoModel.fromJson(Map<String, dynamic> jsonMap) {
    return PatientDetailInfoModel().._loadFromMap(jsonMap);
  }

  Map<String, dynamic> toJson() {
    final jsonMap = <String, dynamic>{};
    jsonMap["workUnit"] = workUnit;
    jsonMap["contactName"] = contactName;
    jsonMap["contactPhone"] = contactPhone;
    jsonMap["permanentlyResideType"] = permanentlyResideType;
    jsonMap["bloodType"] = bloodType;
    jsonMap["rh"] = rh;
    jsonMap["educationLevel"] = educationLevel;
    jsonMap["career"] = career;
    jsonMap["maritalStatus"] = maritalStatus;
    jsonMap["providerPayments"] = providerPayments;
    jsonMap["providerPaymentsOther"] = providerPaymentsOther;
    return jsonMap;
  }

  void _loadFromMap(Map<String, dynamic> jsonMap) {
    final e = jsonMap;
    workUnit = e["workUnit"];
    contactName = e["contactName"];
    contactPhone = e["contactPhone"];
    permanentlyResideType = e["permanentlyResideType"];
    bloodType = e["bloodType"];
    rh = e["rh"];
    educationLevel = e["educationLevel"];
    career = e["career"];
    maritalStatus = e["maritalStatus"];
    providerPayments = e["providerPayments"];
    providerPaymentsOther = e["providerPaymentsOther"];
  }
}

class PatientHealthInfoModel {
  PatientHealthInfoModel();

  List<String>? historyDrugAllergies;
  List<String>? historyExposure;
  List<String>? historyPreviousDisease;
  String? historyPreviousDiseaseOther;

  ///既往疾病史-职业病
  String? historyPreviousDiseaseOccupationalDisease;

  ///既往疾病史-恶性肿瘤
  String? historyPreviousDiseaseMalignancy;
  String? operation;
  String? trauma;
  String? transfusion;
  List<String>? father;
  String? fatherOther;
  List<String>? mother;
  String? motherOther;
  List<String>? sibling;
  String? siblingOther;
  List<String>? children;
  String? childrenOther;
  String? historyGenetic;
  String? historyGeneticName;
  List<String>? disabilitySituation;
  String? disabilitySituationOther;
  String? kitchenExhaustFacilities;
  String? fuel;
  String? fuelOther;
  String? drinkingWater;
  String? drinkingWaterOther;
  String? toilet;
  String? animalPen;
  String? historyDrugAllergiesOther;
  VProfileTimeInputModel? operationOne;
  VProfileTimeInputModel? operationTwo;
  VProfileTimeInputModel? traumaOne;
  VProfileTimeInputModel? traumaTwo;
  VProfileTimeInputModel? transfusionOne;
  VProfileTimeInputModel? transfusionTwo;
  Map<String, DateTime> diseaseDiagnosisTime = {};

  factory PatientHealthInfoModel.fromJson(Map<String, dynamic> jsonMap) {
    return PatientHealthInfoModel().._loadFromMap(jsonMap);
  }

  Map<String, dynamic> toJson() {
    final jsonMap = <String, dynamic>{};
    if (historyDrugAllergies != null) {
      jsonMap["historyDrugAllergies"] = historyDrugAllergies;
    }
    if (historyDrugAllergies?.contains('其他') == true) {
      historyDrugAllergiesOther = '';
    }
    if (historyDrugAllergiesOther != null) {
      jsonMap["historyDrugAllergiesOther"] = historyDrugAllergiesOther;
    }
    if (historyExposure != null) jsonMap["historyExposure"] = historyExposure;
    if (historyPreviousDisease != null) {
      jsonMap["historyPreviousDisease"] = historyPreviousDisease;
    }
    if (historyPreviousDisease?.contains('其他') == false) {
      historyPreviousDiseaseOther = '';
    }
    if (historyPreviousDisease?.contains('职业病') == false) {
      historyPreviousDiseaseOther = '';
    }
    if (historyPreviousDisease?.contains('恶性肿瘤') == false) {
      historyPreviousDiseaseOther = '';
    }
    for (var key in diseaseDiagnosisTime.keys) {
      if (historyPreviousDisease?.contains(key) == false) {
        diseaseDiagnosisTime.remove(key);
      }
    }
    if (historyPreviousDiseaseOther != null) {
      jsonMap["historyPreviousDiseaseOther"] = historyPreviousDiseaseOther;
    }
    if (historyPreviousDiseaseOccupationalDisease != null) {
      jsonMap["historyPreviousDiseaseOccupationalDisease"] =
          historyPreviousDiseaseOccupationalDisease;
    }
    if (historyPreviousDiseaseMalignancy != null) {
      jsonMap["historyPreviousDiseaseMalignancy"] =
          historyPreviousDiseaseMalignancy;
    }
    if (operation != null) jsonMap["operation"] = operation;
    if (operation != "2") {
      operationOne = null;
      operationTwo = null;
    }
    if (trauma != null) jsonMap["trauma"] = trauma;
    if (trauma != "2") {
      traumaOne = null;
      traumaTwo = null;
    }
    if (transfusion != null) jsonMap["transfusion"] = transfusion;
    if (transfusion != "2") {
      transfusionOne = null;
      transfusionTwo = null;
    }

    if (father != null) jsonMap["father"] = father;
    if (father?.contains('其他') == false) {
      fatherOther = null;
    }
    if (fatherOther != null) jsonMap["fatherOther"] = fatherOther;
    if (mother != null) jsonMap["mother"] = mother;
    if (mother?.contains('其他') == false) {
      motherOther = null;
    }
    if (motherOther != null) jsonMap["motherOther"] = motherOther;
    if (sibling != null) jsonMap["sibling"] = sibling;
    if (sibling?.contains('其他') == false) {
      siblingOther = null;
    }
    if (siblingOther != null) jsonMap["siblingOther"] = siblingOther;
    if (children != null) jsonMap["children"] = children;
    if (children?.contains('其他') == false) {
      childrenOther = null;
    }
    if (childrenOther != null) jsonMap["childrenOther"] = childrenOther;
    if (historyGenetic != null) jsonMap["historyGenetic"] = historyGenetic;
    if (historyGenetic != '2') {
      historyGeneticName = '';
    }
    if (historyGeneticName != null) {
      jsonMap["historyGeneticName"] = historyGeneticName;
    }
    if (disabilitySituation != null) {
      jsonMap["disabilitySituation"] = disabilitySituation;
    }
    if (disabilitySituation?.contains('8') == false) {
      disabilitySituationOther = '';
    }
    if (disabilitySituationOther != null) {
      jsonMap["disabilitySituationOther"] = disabilitySituationOther;
    }
    if (kitchenExhaustFacilities != null) {
      jsonMap["kitchenExhaustFacilities"] = kitchenExhaustFacilities;
    }
    if (fuel != null) jsonMap["fuel"] = fuel;
    if (fuel != '6') {
      fuelOther = '';
    }
    if (fuelOther != null) jsonMap["fuelOther"] = fuelOther;
    if (drinkingWater != null) jsonMap["drinkingWater"] = drinkingWater;
    if (drinkingWater != '6') {
      drinkingWaterOther = '';
    }
    if (drinkingWaterOther != null) {
      jsonMap["drinkingWaterOther"] = drinkingWaterOther;
    }
    if (toilet != null) jsonMap["toilet"] = toilet;
    if (animalPen != null) jsonMap["animalPen"] = animalPen;
    if (operationOne != null) {
      jsonMap["operationOne"] = jsonEncode(operationOne?.toJson());
    }
    if (operationTwo != null) {
      jsonMap["operationTwo"] = jsonEncode(operationTwo?.toJson());
    }
    if (traumaOne != null) {
      jsonMap["traumaOne"] = jsonEncode(traumaOne?.toJson());
    }
    if (traumaTwo != null) {
      jsonMap["traumaTwo"] = jsonEncode(traumaTwo?.toJson());
    }
    if (transfusionOne != null) {
      jsonMap["transfusionOne"] = jsonEncode(transfusionOne?.toJson());
    }
    if (transfusionTwo != null) {
      jsonMap["transfusionTwo"] = jsonEncode(transfusionTwo?.toJson());
    }
    jsonMap["diseaseDiagnosisTime"] = jsonEncode(diseaseDiagnosisTime
        .map((key, value) => MapEntry(key, value.toIso8601String())));

    return jsonMap;
  }

  void _loadFromMap(Map<String, dynamic> jsonMap) {
    final e = jsonMap;
    historyDrugAllergies = e["historyDrugAllergies"]?.cast<String>().toList();
    historyExposure = e["historyExposure"]?.cast<String>().toList();
    historyPreviousDisease =
        e["historyPreviousDisease"]?.cast<String>().toList();
    historyPreviousDiseaseOther = e["historyPreviousDiseaseOther"];
    historyPreviousDiseaseOccupationalDisease =
        e["historyPreviousDiseaseOccupationalDisease"];
    historyPreviousDiseaseMalignancy = e["historyPreviousDiseaseMalignancy"];
    operation = e["operation"];
    trauma = e["trauma"];
    transfusion = e["transfusion"];
    father = e["father"]?.cast<String>().toList();
    fatherOther = e["fatherOther"];
    mother = e["mother"]?.cast<String>().toList();
    motherOther = e["motherOther"];
    sibling = e["sibling"]?.cast<String>().toList();
    siblingOther = e["siblingOther"];
    children = e["children"]?.cast<String>().toList();
    childrenOther = e["childrenOther"];
    historyGenetic = e["historyGenetic"];
    historyGeneticName = e["historyGeneticName"];
    disabilitySituation = e["disabilitySituation"]?.cast<String>().toList();
    disabilitySituationOther = e["disabilitySituationOther"];
    kitchenExhaustFacilities = e["kitchenExhaustFacilities"];
    fuel = e["fuel"];
    fuelOther = e["fuelOther"];
    drinkingWater = e["drinkingWater"];
    drinkingWaterOther = e["drinkingWaterOther"];
    toilet = e["toilet"];
    animalPen = e["animalPen"];
    historyDrugAllergiesOther = e["historyDrugAllergiesOther"];
    operationOne = e["operationOne"] != null && e["operationOne"] != "{}"
        ? VProfileTimeInputModel.fromJson(jsonDecode(e["operationOne"]))
        : null;
    operationTwo = e["operationTwo"] != null && e["operationTwo"] != "{}"
        ? VProfileTimeInputModel.fromJson(jsonDecode(e["operationTwo"]))
        : null;
    traumaOne = e["traumaOne"] != null && e["traumaOne"] != "{}"
        ? VProfileTimeInputModel.fromJson(jsonDecode(e["traumaOne"]))
        : null;
    traumaTwo = e["traumaTwo"] != null && e["traumaTwo"] != "{}"
        ? VProfileTimeInputModel.fromJson(jsonDecode(e["traumaTwo"]))
        : null;
    transfusionOne = e["transfusionOne"] != null && e["transfusionOne"] != "{}"
        ? VProfileTimeInputModel.fromJson(jsonDecode(e["transfusionOne"]))
        : null;
    transfusionTwo = e["transfusionTwo"] != null && e["transfusionTwo"] != "{}"
        ? VProfileTimeInputModel.fromJson(jsonDecode(e["transfusionTwo"]))
        : null;
    diseaseDiagnosisTime =
        Map<String, dynamic>.from(jsonDecode(e["diseaseDiagnosisTime"]))
            .map((key, value) {
      return MapEntry(key, DateTime.parse(value));
    });
  }
}
