import 'package:get/get.dart';
import 'package:vitalapp/architecture/defines.dart';
import 'package:vitalapp/global.dart';
import 'package:vitalapp/managers/interfaces/models/patient_model_dto.dart';
import 'package:vitalapp/pages/controllers/paged_state_mixin.dart';

class PatientListState with PagedStateMixin<PatientModelDTO> {
  static int _statisticTotalCountCache = 0;
  static int _statisticTodayCountCache = 0;

  PatientListState() {
    if (!kIsOnline) {
      _statisticTotalCount.value = _statisticTotalCountCache;
      _statisticTodayCount.value = _statisticTodayCountCache;
    }
  }

  final _now = DateTime.now();

  final RxInt _currentTabIndex = RxInt(0);
  final RxList<String> _offlineCodes = RxList<String>();
  final RxInt _statisticTotalCount = RxInt(0);
  final RxInt _statisticTodayCount = RxInt(0);

  int get currentTabIndex => _currentTabIndex.value;
  set currentTabIndex(int val) => _currentTabIndex.updateValue(val);

  /// 离线code列表
  List<String> get offlineCodes => _offlineCodes.toList();
  set offlineCodes(List<String> val) => _offlineCodes.value = val;

  /// 离线数量
  int get offlineCount => offlineCodes.length;

  /// 查询结果总数
  int get queryResultTotalCount => offlineCount + totalCount;

  /// 居民总数
  int get statisticTotalCount => _statisticTotalCount.value;
  set statisticTotalCount(int val) {
    _statisticTotalCount.updateValue(val);
    _statisticTotalCountCache = val;
  }

  /// 今日新增数
  int get statisticTodayCount => _statisticTodayCount.value;
  set statisticTodayCount(int val) {
    _statisticTodayCount.updateValue(val);
    _statisticTodayCountCache = val;
  }

  late Rx<DateTime?> startTime =
      DateTime(_now.year - 1, _now.month, _now.day).obs; // >= 1年前00:00:00
  late Rx<DateTime?> endTime =
      DateTime(_now.year, _now.month, _now.day).obs; // < 第二天00:00:00
}
