import 'package:fis_common/logger/logger.dart';
import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:get/get.dart';
import 'package:vitalapp/architecture/utils/prompt_box.dart';
import 'package:vitalapp/managers/interfaces/models/report_tag_key.dart';
import 'package:vitalapp/pages/report_edit/controller.dart';
import 'package:vitalapp/pages/report_edit/state.dart';
import 'package:vitalapp/pages/report_edit/widgets/add_report_tag.dart';
import 'package:vitalapp/rpc.dart';
import 'package:vitalapp/store/store.dart';

class AddReportTagController {
  late final ReportEditController controller;
  late final ReportEditState state;

  AddReportTagController(ReportEditController controller) {
    this.controller = controller;
    this.state = controller.state;
  }

  ///打开添加报告标签弹窗
  Future<void> openAddReportTag() async {
    ///是否需要打开判断
    await Get.dialog(AddReportTag(
        state.allReportTags, state.selectedTags, submitReportTags));
  }

  ///提交报告标签
  Future<void> submitReportTags(List<String> values) async {
    try {
      final result =
          await rpc.report.modifyReportLabelsAsync(ModifyReportLabelsRequest(
        token: Store.user.token,
        reportCode: state.reportCode,
        reportLabels: values,
      ));
      if (result) {
        PromptBox.toast(i18nBook.remedical.submitSuccess.t);
        Future.delayed(Duration(milliseconds: 500), () {
          Get.back();
        });
      } else {
        PromptBox.toast(i18nBook.remedical.submitFail.t);
      }
    } catch (e) {
      logger.e('AddReportTagController submitReportTags ex:', e);
    }
  }
}
