import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:vitalapp/architecture/utils/prompt_box.dart';
import 'package:vitalapp/components/alert_dialog.dart';
import 'package:vitalapp/pages/settings/server/controller.dart';
import 'package:vitalapp/store/store.dart';

class SavedServersDialog extends StatelessWidget {
  const SavedServersDialog({super.key});

  static Future<String?> show() async {
    final result = await Get.dialog(const SavedServersDialog());
    return result;
  }

  @override
  Widget build(BuildContext context) {
    return VAlertDialog(
      title: "已保存服务地址列表",
      width: 560,
      content: SizedBox(
        height: 320,
        child: _buildBody(context),
      ),
    );
  }

  Widget _buildBody(BuildContext context) {
    final controller = Get.find<ServerSettingController>();
    return Obx(
      () => ListView.builder(
        padding: const EdgeInsets.symmetric(horizontal: 18),
        itemCount: controller.state.savedList.length,
        itemBuilder: (_, index) {
          final url = controller.state.savedList[index];
          final inUse = url == Store.app.serverUrl;
          return _ListItem(
            url: url,
            inUse: inUse,
            onSelected: () {
              controller.switchSavedServer(index);
            },
            onDeleted: () {
              controller.removeSavedServer(index);
            },
          );
        },
      ),
    );
  }
}

class _ListItem extends StatelessWidget {
  final String url;
  final bool inUse;

  final VoidCallback onSelected;
  final VoidCallback onDeleted;

  const _ListItem({
    required this.url,
    required this.onSelected,
    required this.onDeleted,
    this.inUse = false,
  });

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: const EdgeInsets.symmetric(horizontal: 18, vertical: 6),
      decoration: BoxDecoration(
        border: Border(
          bottom: BorderSide(color: Colors.grey.shade300),
        ),
      ),
      child: Row(
        children: [
          Expanded(
            child: Text(url, style: const TextStyle(fontSize: 18)),
          ),
          const SizedBox(width: 12),
          SizedBox(
            width: 160,
            height: 46,
            child: inUse
                ? const Center(
                    child: Text(
                      "正在使用中",
                      style: TextStyle(color: Colors.grey, fontSize: 14),
                    ),
                  )
                : Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      TextButton(
                        onPressed: () {
                          onSelected.call();
                        },
                        child: Text(
                          "选择",
                          style: TextStyle(
                            color: Theme.of(context).primaryColor,
                            fontSize: 14,
                          ),
                        ),
                      ),
                      const SizedBox(width: 2),
                      TextButton(
                        onPressed: () {
                          onDeleted.call();
                        },
                        child: const Text(
                          "移除",
                          style: TextStyle(
                            color: Colors.red,
                            fontSize: 14,
                          ),
                        ),
                      ),
                    ],
                  ),
          ),
          // TextButton(
          //   onPressed: () {
          //     onSelected.call();
          //   },
          //   child: Text(
          //     "选择",
          //     style: TextStyle(
          //       color: Theme.of(context).primaryColor,
          //       fontSize: 14,
          //     ),
          //   ),
          // ),
          // const SizedBox(width: 2),
          // TextButton(
          //   onPressed: inUse
          //       ? () {
          //           PromptBox.toast("此地址正在使用中，不可移除！");
          //         }
          //       : () {
          //           onDeleted.call();
          //         },
          //   child: Text(
          //     "移除",
          //     style: TextStyle(
          //       color: inUse ? Colors.grey : Colors.red,
          //       fontSize: 14,
          //     ),
          //   ),
          // ),
        ],
      ),
    );
  }
}
